/*
 *************************************************************************
 * Ralink Tech Inc.
 * 5F., No.36, Taiyuan St., Jhubei City,
 * Hsinchu County 302,
 * Taiwan, R.O.C.
 *
 * (c) Copyright 2002-2007, Ralink Technology, Inc.
 *
 * This program is free software; you can redistribute it and/or modify  * 
 * it under the terms of the GNU General Public License as published by  * 
 * the Free Software Foundation; either version 2 of the License, or     * 
 * (at your option) any later version.                                   * 
 *                                                                       * 
 * This program is distributed in the hope that it will be useful,       * 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         * 
 * GNU General Public License for more details.                          * 
 *                                                                       * 
 * You should have received a copy of the GNU General Public License     * 
 * along with this program; if not, write to the                         * 
 * Free Software Foundation, Inc.,                                       * 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             * 
 *                                                                       * 
 *************************************************************************

    Module Name:
    eeprom.c

    Abstract:

    Revision History:
    Who         When          What
    --------    ----------    ----------------------------------------------
	Eddy Tsai	01-12-2008    Created
*/
#include "rt_config.h"


void RTMPusecDelay(ULONG usec)
{
	ULONG	i;

	for (i = 0; i < (usec / 50); i++)
		udelay(50);

	if (usec % 50)
		udelay(usec % 50);
} /* End of RTMPusecDelay */


static void RaiseClock(PRTMP_ADAPTER pAd, UINT32 *x)
{
    *x = *x | EESK;
    RTMP_IO_WRITE32(pAd, E2PROM_CSR, *x);
    RTMPusecDelay(1);
} /* End of RaiseClock */


static void LowerClock(PRTMP_ADAPTER pAd, UINT32 *x)
{
    *x = *x & ~EESK;
    RTMP_IO_WRITE32(pAd, E2PROM_CSR, *x);
    RTMPusecDelay(1);
} /* End of LowerClock */


static USHORT ShiftInBits(PRTMP_ADAPTER pAd)
{
    UINT32	x,i;
	USHORT  data=0;

    RTMP_IO_READ32(pAd, E2PROM_CSR, &x);
    x &= ~(EEDO | EEDI);
    for(i=0; i<16; i++)
    {
        data = data << 1;
        RaiseClock(pAd, &x);
        RTMP_IO_READ32(pAd, E2PROM_CSR, &x);
        x &= ~(EEDI);
        if(x & EEDO)
            data |= 1;
        LowerClock(pAd, &x);
    }

    return data;
} /* End of ShiftInBits */


static void ShiftOutBits(PRTMP_ADAPTER	pAd, USHORT data,
		USHORT count)
{
    UINT32	x,mask;

    mask = 0x01 << (count - 1);
    RTMP_IO_READ32(pAd, E2PROM_CSR, &x);

    x &= ~(EEDO | EEDI);
    do
    {
        x &= ~EEDI;
        if (data & mask)
			x |= EEDI;
        RTMP_IO_WRITE32(pAd, E2PROM_CSR, x);
        RaiseClock(pAd, &x);
        LowerClock(pAd, &x);
        mask = mask >> 1;
    } while(mask);

    x &= ~EEDI;
    RTMP_IO_WRITE32(pAd, E2PROM_CSR, x);
} /* End of ShiftOutBits */


static void EEpromCleanup(PRTMP_ADAPTER pAd)
{
    UINT32  x;

    RTMP_IO_READ32(pAd, E2PROM_CSR, &x);
    x &= ~(EECS | EEDI);
    RTMP_IO_WRITE32(pAd, E2PROM_CSR, x);
    RaiseClock(pAd, &x);
    LowerClock(pAd, &x);	
} /* End of EEpromCleanup */


static USHORT RTMP_EEPROM_READ16(PRTMP_ADAPTER	pAd,
		USHORT offset)
{
    UINT32	x;
    USHORT	data;

    offset /= 2;
    // reset bits and set EECS
    RTMP_IO_READ32(pAd, E2PROM_CSR, &x);
    x &= ~(EEDI | EEDO | EESK);
    x |= EECS;
    RTMP_IO_WRITE32(pAd, E2PROM_CSR, x);

	// kick a pulse
	RaiseClock(pAd, &x);
	LowerClock(pAd, &x);

    // output the read_opcode and register number in that order    
    ShiftOutBits(pAd, EEPROM_READ_OPCODE, 3);
    ShiftOutBits(pAd, offset, pAd->EEPROMAddressNum);

    // Now read the data (16 bits) in from the selected EEPROM word
    data = ShiftInBits(pAd);

    EEpromCleanup(pAd);

    return data;
} /* End of RTMP_EEPROM_READ16 */


void NICReadEEPROMParameters(PRTMP_ADAPTER	pAd, UCHAR *mac_addr)
{
	UINT32			data = 0;
	USHORT			value;	
	USHORT			Addr01,Addr23,Addr45;
	UINT32			i,choffset;
	EEPROM_TX_PWR_STRUC		Power;
	EEPROM_TX_PWR_STRUC		Power2;
	EEPROM_ANTENNA_STRUC	Antenna;

	// Init EEPROM Address Number, before access EEPROM
	RTMP_IO_READ32(pAd, E2PROM_CSR, &data);
	if((data & 0x30) == 0)
		pAd->EEPROMAddressNum = 6;	// 93C46
	else if((data & 0x30) == 0x10)
		pAd->EEPROMAddressNum = 8;	// 93C66
	else
		pAd->EEPROMAddressNum = 8;	// 93C86

	if (mac_addr == NULL || strlen((char *)mac_addr) != 17 || 
		mac_addr[2] != ':'  || mac_addr[5] != ':'  || mac_addr[8] != ':' ||
		mac_addr[11] != ':' || mac_addr[14] != ':')
	{
		RT28xx_EEPROM_READ16(pAd, 0x04, Addr01);
		RT28xx_EEPROM_READ16(pAd, 0x06, Addr23);
		RT28xx_EEPROM_READ16(pAd, 0x08, Addr45);
		pAd->PermanentAddress[0] = (UCHAR)(Addr01 & 0xff);
		pAd->PermanentAddress[1] = (UCHAR)(Addr01 >> 8);
		pAd->PermanentAddress[2] = (UCHAR)(Addr23 & 0xff);
		pAd->PermanentAddress[3] = (UCHAR)(Addr23 >> 8);
		pAd->PermanentAddress[4] = (UCHAR)(Addr45 & 0xff);
		pAd->PermanentAddress[5] = (UCHAR)(Addr45 >> 8);
	}

	/*
	 * Get RSSI Offset on EEPROM 0x9Ah & 0x9Ch.
	 * The valid value are (-10 ~ 10) 
	 */ 
	RT28xx_EEPROM_READ16(pAd, EEPROM_RSSI_BG_OFFSET, value);
	pAd->BGRssiOffset0 = value & 0x00ff;
	pAd->BGRssiOffset1 = (value >> 8);
	RT28xx_EEPROM_READ16(pAd, EEPROM_RSSI_BG_OFFSET+2, value);
	pAd->BGRssiOffset2 = value & 0x00ff;
	pAd->ALNAGain1 = (value >> 8);
	RT28xx_EEPROM_READ16(pAd, EEPROM_LNA_OFFSET, value);
	pAd->BLNAGain = value & 0x00ff;
	pAd->ALNAGain0 = (value >> 8);
	// Validate 11b/g RSSI_0 offset.
	if ((pAd->BGRssiOffset0 < -10) || (pAd->BGRssiOffset0 > 10))
		pAd->BGRssiOffset0 = 0;
	// Validate 11b/g RSSI_1 offset.
	if ((pAd->BGRssiOffset1 < -10) || (pAd->BGRssiOffset1 > 10))
		pAd->BGRssiOffset1 = 0;
	// Validate 11b/g RSSI_2 offset.
	if ((pAd->BGRssiOffset2 < -10) || (pAd->BGRssiOffset2 > 10))
		pAd->BGRssiOffset2 = 0;
		
	RT28xx_EEPROM_READ16(pAd, EEPROM_RSSI_A_OFFSET, value);
	pAd->ARssiOffset0 = value & 0x00ff;
	pAd->ARssiOffset1 = (value >> 8);
	RT28xx_EEPROM_READ16(pAd, (EEPROM_RSSI_A_OFFSET+2), value);
	pAd->ARssiOffset2 = value & 0x00ff;
	pAd->ALNAGain2 = (value >> 8);
	if (((UCHAR)pAd->ALNAGain1 == 0xFF) || (pAd->ALNAGain1 == 0x00))
		pAd->ALNAGain1 = pAd->ALNAGain0;
	if (((UCHAR)pAd->ALNAGain2 == 0xFF) || (pAd->ALNAGain2 == 0x00))
		pAd->ALNAGain2 = pAd->ALNAGain0;

	// Validate 11a RSSI_0 offset.
	if ((pAd->ARssiOffset0 < -10) || (pAd->ARssiOffset0 > 10))
		pAd->ARssiOffset0 = 0;
	// Validate 11a RSSI_1 offset.
	if ((pAd->ARssiOffset1 < -10) || (pAd->ARssiOffset1 > 10))
		pAd->ARssiOffset1 = 0;
	//Validate 11a RSSI_2 offset.
	if ((pAd->ARssiOffset2 < -10) || (pAd->ARssiOffset2 > 10))
		pAd->ARssiOffset2 = 0;

	/* Read Tx power value for all channels
	 * Value from 1 - 0x7f. Default value is 24.
	 * Power value : 2.4G 0x00 (0) ~ 0x1F (31)
	 *             : 5.5G 0xF9 (-7) ~ 0x0F (15)
	 */
	/* 0. 11b/g, ch1 - ch 14 */
	for (i = 0; i < 7; i++)
	{
		RT28xx_EEPROM_READ16(pAd, EEPROM_G_TX_PWR_OFFSET + i*2, Power.word);
		RT28xx_EEPROM_READ16(pAd, EEPROM_G_TX2_PWR_OFFSET + i*2, Power2.word);
		pAd->TxPower[i*2].Channel = i*2 + 1;
		pAd->TxPower[i*2 + 1].Channel = i*2 + 2;

		if ((Power.field.Byte0 > 31) || (Power.field.Byte0 < 0))
			pAd->TxPower[i*2].Power = DEFAULT_RF_TX_POWER;
		else
			pAd->TxPower[i*2].Power = Power.field.Byte0;

		if ((Power.field.Byte1 > 31) || (Power.field.Byte1 < 0))
			pAd->TxPower[i*2 + 1].Power = DEFAULT_RF_TX_POWER;
		else
			pAd->TxPower[i*2 + 1].Power = Power.field.Byte1;

		if ((Power2.field.Byte0 > 31) || (Power2.field.Byte0 < 0))
			pAd->TxPower[i*2].Power2 = DEFAULT_RF_TX_POWER;
		else
			pAd->TxPower[i*2].Power2 = Power2.field.Byte0;

		if ((Power2.field.Byte1 > 31) || (Power2.field.Byte1 < 0))
			pAd->TxPower[i*2 + 1].Power2 = DEFAULT_RF_TX_POWER;
		else
			pAd->TxPower[i*2 + 1].Power2 = Power2.field.Byte1;
	}	
	/* 1. U-NII lower/middle band: 36, 38, 40; 44, 46, 48; 52, 54, 56; 60, 62, 64 (including central frequency in BW 40MHz) */
	/* 1.1 Fill up channel */
	choffset = 14;
	for (i = 0; i < 4; i++)
	{
		pAd->TxPower[3*i + choffset + 0].Channel= 36 + i*8 + 0;
		pAd->TxPower[3*i + choffset + 0].Power	= DEFAULT_RF_TX_POWER;
		pAd->TxPower[3*i + choffset + 0].Power2	= DEFAULT_RF_TX_POWER;
		pAd->TxPower[3*i + choffset + 1].Channel= 36 + i*8 + 2;
		pAd->TxPower[3*i + choffset + 1].Power	= DEFAULT_RF_TX_POWER;
		pAd->TxPower[3*i + choffset + 1].Power2	= DEFAULT_RF_TX_POWER;
		pAd->TxPower[3*i + choffset + 2].Channel= 36 + i*8 + 4;
		pAd->TxPower[3*i + choffset + 2].Power	= DEFAULT_RF_TX_POWER;
		pAd->TxPower[3*i + choffset + 2].Power2	= DEFAULT_RF_TX_POWER;
	}

	/* 1.2 Fill up power */
	for (i = 0; i < 6; i++)
	{
		RT28xx_EEPROM_READ16(pAd, EEPROM_A_TX_PWR_OFFSET + i*2, Power.word);
		RT28xx_EEPROM_READ16(pAd, EEPROM_A_TX2_PWR_OFFSET + i*2, Power2.word);

		if ((Power.field.Byte0 < 16) && (Power.field.Byte0 >= -7))
			pAd->TxPower[i*2 + choffset + 0].Power = Power.field.Byte0;

		if ((Power.field.Byte1 < 16) && (Power.field.Byte1 >= -7))
			pAd->TxPower[i*2 + choffset + 1].Power = Power.field.Byte1;			

		if ((Power2.field.Byte0 < 16) && (Power2.field.Byte0 >= -7))
			pAd->TxPower[i*2 + choffset + 0].Power2 = Power2.field.Byte0;

		if ((Power2.field.Byte1 < 16) && (Power2.field.Byte1 >= -7))
			pAd->TxPower[i*2 + choffset + 1].Power2 = Power2.field.Byte1;			
	}
	/* 2. HipperLAN 2 100, 102 ,104; 108, 110, 112; 116, 118, 120; 124, 126, 128; 132, 134, 136; 140 (including central frequency in BW 40MHz) */
	/* 2.1 Fill up channel */
	choffset = 14 + 12;
	for (i = 0; i < 5; i++)
	{
		pAd->TxPower[3*i + choffset + 0].Channel= 100 + i*8 + 0;
		pAd->TxPower[3*i + choffset + 0].Power	= DEFAULT_RF_TX_POWER;
		pAd->TxPower[3*i + choffset + 0].Power2	= DEFAULT_RF_TX_POWER;
		pAd->TxPower[3*i + choffset + 1].Channel= 100 + i*8 + 2;
		pAd->TxPower[3*i + choffset + 1].Power	= DEFAULT_RF_TX_POWER;
		pAd->TxPower[3*i + choffset + 1].Power2	= DEFAULT_RF_TX_POWER;
		pAd->TxPower[3*i + choffset + 2].Channel= 100 + i*8 + 4;
		pAd->TxPower[3*i + choffset + 2].Power	= DEFAULT_RF_TX_POWER;
		pAd->TxPower[3*i + choffset + 2].Power2	= DEFAULT_RF_TX_POWER;
	}
	pAd->TxPower[3*5 + choffset + 0].Channel	= 140;
	pAd->TxPower[3*5 + choffset + 0].Power	= DEFAULT_RF_TX_POWER;
	pAd->TxPower[3*5 + choffset + 0].Power2	= DEFAULT_RF_TX_POWER;
	/* 2.2 Fill up power */
	for (i = 0; i < 8; i++)
	{
		RT28xx_EEPROM_READ16(pAd, 
			EEPROM_A_TX_PWR_OFFSET + (choffset - 14) + i*2, Power.word);
		RT28xx_EEPROM_READ16(pAd, 
			EEPROM_A_TX2_PWR_OFFSET + (choffset - 14) + i*2, Power2.word);

		if ((Power.field.Byte0 < 16) && (Power.field.Byte0 >= -7))
			pAd->TxPower[i*2 + choffset + 0].Power = Power.field.Byte0;

		if ((Power.field.Byte1 < 16) && (Power.field.Byte1 >= -7))
			pAd->TxPower[i*2 + choffset + 1].Power = Power.field.Byte1;			

		if ((Power2.field.Byte0 < 16) && (Power2.field.Byte0 >= -7))
			pAd->TxPower[i*2 + choffset + 0].Power2 = Power2.field.Byte0;

		if ((Power2.field.Byte1 < 16) && (Power2.field.Byte1 >= -7))
			pAd->TxPower[i*2 + choffset + 1].Power2 = Power2.field.Byte1;			
	}

	/* 3. U-NII upper band: 149, 151, 153; 157, 159, 161; 165 (including central frequency in BW 40MHz) */
	/* 3.1 Fill up channel */
	choffset = 14 + 12 + 16;
	for (i = 0; i < 2; i++)
	{
		pAd->TxPower[3*i + choffset + 0].Channel= 149 + i*8 + 0;
		pAd->TxPower[3*i + choffset + 0].Power	= DEFAULT_RF_TX_POWER;
		pAd->TxPower[3*i + choffset + 0].Power2	= DEFAULT_RF_TX_POWER;

		pAd->TxPower[3*i + choffset + 1].Channel= 149 + i*8 + 2;
		pAd->TxPower[3*i + choffset + 1].Power	= DEFAULT_RF_TX_POWER;
		pAd->TxPower[3*i + choffset + 1].Power2	= DEFAULT_RF_TX_POWER;

		pAd->TxPower[3*i + choffset + 2].Channel= 149 + i*8 + 4;
		pAd->TxPower[3*i + choffset + 2].Power	= DEFAULT_RF_TX_POWER;
		pAd->TxPower[3*i + choffset + 2].Power2	= DEFAULT_RF_TX_POWER;
	}
	pAd->TxPower[3 * 2 + choffset + 0].Channel	= 165;
	pAd->TxPower[3 * 2 + choffset + 0].Power	= DEFAULT_RF_TX_POWER;
	pAd->TxPower[3 * 2 + choffset + 0].Power2	= DEFAULT_RF_TX_POWER;

	/* 3.2 Fill up power */
	for (i = 0; i < 4; i++)
	{
		RT28xx_EEPROM_READ16(pAd, 
			EEPROM_A_TX_PWR_OFFSET + (choffset - 14) + i*2, Power.word);
		RT28xx_EEPROM_READ16(pAd, 
			EEPROM_A_TX2_PWR_OFFSET + (choffset - 14) + i*2, Power2.word);

		if ((Power.field.Byte0 < 16) && (Power.field.Byte0 >= -7))
			pAd->TxPower[i * 2 + choffset + 0].Power = Power.field.Byte0;

		if ((Power.field.Byte1 < 16) && (Power.field.Byte1 >= -7))
			pAd->TxPower[i * 2 + choffset + 1].Power = Power.field.Byte1;			

		if ((Power2.field.Byte0 < 16) && (Power2.field.Byte0 >= -7))
			pAd->TxPower[i * 2 + choffset + 0].Power2 = Power2.field.Byte0;

		if ((Power2.field.Byte1 < 16) && (Power2.field.Byte1 >= -7))
			pAd->TxPower[i * 2 + choffset + 1].Power2 = Power2.field.Byte1;			
	}

	/* Read BBP default value from EEPROM */
	RT28xx_EEPROM_READ16(pAd, EEPROM_NIC1_OFFSET, value);
	pAd->EEPROMDefaultValue[0] = value;

	RT28xx_EEPROM_READ16(pAd, EEPROM_NIC2_OFFSET, value);
	pAd->EEPROMDefaultValue[1] = value;

	RT28xx_EEPROM_READ16(pAd, 0x38, value);	// Country Region
	pAd->EEPROMDefaultValue[2] = value;

	Antenna.word = pAd->EEPROMDefaultValue[0];
	if (Antenna.word == 0xFFFF)
	{
		Antenna.word = 0;
		Antenna.field.RfIcType = RFIC_2820;
		Antenna.field.TxPath = 1;
		Antenna.field.RxPath = 2;
	}
	pAd->Antenna.word = Antenna.word;	
	if (Antenna.field.RfIcType == RFIC_2820) {
		pAd->PhyMode = PHY_11BG_MIXED;
	} else if (Antenna.field.RfIcType == RFIC_2850) {
		pAd->PhyMode = PHY_11ABG_MIXED;
	} else {
		printk(KERN_ERR "Not Known RFIC Type %d\n", Antenna.field.RfIcType);
		}
} /* End of NICReadEEPROMParameters */

