/*
 *************************************************************************
 * Ralink Tech Inc.
 * 5F., No.36, Taiyuan St., Jhubei City,
 * Hsinchu County 302,
 * Taiwan, R.O.C.
 *
 * (c) Copyright 2002-2007, Ralink Technology, Inc.
 *
 * This program is free software; you can redistribute it and/or modify  * 
 * it under the terms of the GNU General Public License as published by  * 
 * the Free Software Foundation; either version 2 of the License, or     * 
 * (at your option) any later version.                                   * 
 *                                                                       * 
 * This program is distributed in the hope that it will be useful,       * 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         * 
 * GNU General Public License for more details.                          * 
 *                                                                       * 
 * You should have received a copy of the GNU General Public License     * 
 * along with this program; if not, write to the                         * 
 * Free Software Foundation, Inc.,                                       * 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             * 
 *                                                                       * 
 *************************************************************************

    Module Name:
    rt2860mac.h

    Abstract:
	Ralink RT2860 PCI driver based on mac80211 stack

    Revision History:
    Who         When          What
    --------    ----------    ----------------------------------------------
	Eddy Tsai	01-12-2008    Created
*/
#include "rt2860pci.h"

/* Debug information levels */
LONG DEBUG_TX_Level = DEBUG_ERROR;
LONG DEBUG_RX_Level = DEBUG_ERROR;
LONG DEBUG_MAC_Level = DEBUG_ERROR;

/* Module infomation */
MODULE_AUTHOR("Eddy Tsai <eddy.tsai@ralinktech.com.tw>");
MODULE_DESCRIPTION("Ralink RT2860 Wireless Lan Linux Driver");
MODULE_SUPPORTED_DEVICE(DEVICE_NAME);
MODULE_LICENSE("GPL");
#ifdef MODULE_VERSION
MODULE_VERSION(DRIVER_VERSION);
#endif
/* Ralink PCI device table, include all supported chipsets */
static struct pci_device_id rt2860_pci_tbl[] __devinitdata =
{
	{PCI_DEVICE(RT_PCI_VENDOR_ID, RT2860_PCI_DEVICE_ID)},
	{PCI_DEVICE(RT_PCI_VENDOR_ID, RT2860_PCIe_DEVICE_ID)},
    {0,} /* terminate list */
};
MODULE_DEVICE_TABLE(pci, rt2860_pci_tbl);

/* PCI stack related functions */
static int __devinit rt2860_probe(struct pci_dev *pdev,
				const struct pci_device_id *id);
static void __devexit rt2860_remove(struct pci_dev *pdev);
static struct pci_driver rt2860_pci_driver = {
	.name		= "rt2860_pci",
	.id_table	= rt2860_pci_tbl,
	.probe		= rt2860_probe,
	.remove		= __devexit_p(rt2860_remove),
};


/* Callbacks from mac80211 */
static int rt2860_tx(struct ieee80211_hw *hw, 
				struct sk_buff *skb,
				struct ieee80211_tx_control *control);
static int rt2860_start(struct ieee80211_hw *hw);
static void rt2860_stop(struct ieee80211_hw *hw);
static int rt2860_add_interface(struct ieee80211_hw *hw,
				struct ieee80211_if_init_conf *conf);
static void rt2860_remove_interface(struct ieee80211_hw *hw,
				struct ieee80211_if_init_conf *conf);
static int rt2860_config(struct ieee80211_hw *hw, struct ieee80211_conf *conf);
static int rt2860_config_interface(struct ieee80211_hw *hw,
				struct ieee80211_vif *vif,
				struct ieee80211_if_conf *conf);
static void rt2860_configure_filter(struct ieee80211_hw *hw, 
				unsigned int changed_flags,
				unsigned int *total_flags, 
				int mc_count, struct dev_mc_list *mclist);
static int rt2860_get_stats(struct ieee80211_hw *hw,
			    struct ieee80211_low_level_stats *stats);
static u64 rt2860_get_tsft(struct ieee80211_hw *hw);
static const struct ieee80211_ops rt2860_ops = {
	.tx					= rt2860_tx,
	.start				= rt2860_start,
	.stop				= rt2860_stop,
	.add_interface		= rt2860_add_interface,
	.remove_interface	= rt2860_remove_interface,
	.config				= rt2860_config,
	.config_interface	= rt2860_config_interface,
	.configure_filter	= rt2860_configure_filter,
	.get_stats			= rt2860_get_stats,
	.get_tsf			= rt2860_get_tsft,
};


/* 
 *  Allocate Tx and Rx ring 
 */
static int rt2860_init_rings(struct ieee80211_hw *hw)
{
	RTMP_ADAPTER *pAd = hw->priv;
	INT32			Status = NDIS_STATUS_SUCCESS;
	INT32			TxRingIndex, TxRingNum, MgmtRingIndex, RxRingIndex;
	ULONG			RingBasePaHigh;
	ULONG			RingBasePaLow;
	void			*RingBaseVa;
	PRTMP_DMABUF	pDmaBuf;
	PTXD_STRUC		pTxD;
	PRXD_STRUC		pRxD;
	void 			*pPacket;
	RTMP_DMABUF    *pTxDescRing;
	RTMP_TX_RING   *pTxRing;   
	RTMP_DMABUF    *pRxDescRing;
	RTMP_RX_RING   *pRxRing;   
	RTMP_DMABUF    *pMgmtDescRing;
	RTMP_MGMT_RING *pMgmtRing;   

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_init_rings\n"));

	/* 5 TX rings = 4 ACs + 1 HCCA */
	for (TxRingNum=0; TxRingNum < NUM_OF_TX_RING; TxRingNum++) 
	{
		/* TX Ring */
		pTxDescRing = &pAd->TxDescRing[TxRingNum];
		pTxDescRing->AllocSize = TX_RING_SIZE * TXD_SIZE;
		pTxDescRing->AllocVa = (void *) pci_alloc_consistent(pAd->pdev, 
			sizeof(char)*pTxDescRing->AllocSize, &pTxDescRing->AllocPa);
		if (pTxDescRing->AllocVa == NULL)
		{
			DBGPRINT_MAC(DEBUG_ERROR, 
				("TxDescRing Failed to allocate a big buffer\n"));
			return Status = NDIS_STATUS_FAILURE;
		}
		memset(pTxDescRing->AllocVa, 0,pTxDescRing->AllocSize);
		/* Save PA & VA for further operation */
		RingBasePaHigh = 0;
		RingBasePaLow  = pTxDescRing->AllocPa;
		RingBaseVa     = pTxDescRing->AllocVa;

		pTxRing = &pAd->TxRing[TxRingNum];
		/* Initialize Tx Ring Descriptor and associated buffer memory */
		for (TxRingIndex = 0; TxRingIndex < TX_RING_SIZE; TxRingIndex++)
		{
			pTxRing->Cell[TxRingIndex].pNdisPacket = NULL;
			pTxRing->Cell[TxRingIndex].pNextNdisPacket = NULL;
			/* Init Tx Ring Size, Va, Pa variables */
			pTxRing->Cell[TxRingIndex].AllocSize = TXD_SIZE;
			pTxRing->Cell[TxRingIndex].AllocVa = RingBaseVa;
			pTxRing->Cell[TxRingIndex].AllocPa = RingBasePaLow;

			pTxD = (PTXD_STRUC) pTxRing->Cell[TxRingIndex].AllocVa;
			pTxD->DMADONE = 1;

			RingBasePaLow += TXD_SIZE;
			RingBaseVa += TXD_SIZE;
		} /* End for loop TX_RING_SIZE */
		pTxRing->TxCpuIdx = 0;
		pTxRing->TxDmaIdx = 0;
		pTxRing->TxSwFreeIdx = TX_RING_SIZE - 1;
	} /* End for loop NUM_OF_TX_RING */

	/* Allocate MGMT ring descriptor's memory except Tx ring which allocated eariler */
	pMgmtDescRing = &pAd->MgmtDescRing;
	pMgmtDescRing->AllocSize = MGMT_RING_SIZE * TXD_SIZE;
	pMgmtDescRing->AllocVa = (void *) pci_alloc_consistent(pAd->pdev,
		sizeof(char)*pMgmtDescRing->AllocSize, &pMgmtDescRing->AllocPa);
	if (pMgmtDescRing->AllocVa == NULL)
	{
		DBGPRINT_MAC(DEBUG_ERROR, 
			("pMgmtDescRing Failed to allocate a big buffer\n"));
		return Status = NDIS_STATUS_FAILURE;
	}
	memset(pMgmtDescRing->AllocVa, 0,pMgmtDescRing->AllocSize);
	/* Save PA & VA for further operation */
	RingBasePaHigh = 0;
	RingBasePaLow  = pMgmtDescRing->AllocPa;
	RingBaseVa     = pMgmtDescRing->AllocVa;

	pMgmtRing = &pAd->MgmtRing;
	/* Initialize Mgmt Ring Descriptor and associated buffer memory */
	for (MgmtRingIndex = 0; MgmtRingIndex < MGMT_RING_SIZE; MgmtRingIndex++)
	{
		pMgmtRing->Cell[MgmtRingIndex].pNdisPacket = NULL;
		pMgmtRing->Cell[MgmtRingIndex].pNextNdisPacket = NULL;
		pMgmtRing->Cell[MgmtRingIndex].AllocSize = TXD_SIZE;
		pMgmtRing->Cell[MgmtRingIndex].AllocVa = RingBaseVa;
		pMgmtRing->Cell[MgmtRingIndex].AllocPa = RingBasePaLow;
		pTxD = (PTXD_STRUC) pMgmtRing->Cell[MgmtRingIndex].AllocVa;
		pTxD->DMADONE = 1;

		RingBasePaLow += TXD_SIZE;
		RingBaseVa += TXD_SIZE;
	} /* End for loop MGMT_RING_SIZE */
	pMgmtRing->TxCpuIdx = 0;
	pMgmtRing->TxDmaIdx = 0;
	pMgmtRing->TxSwFreeIdx = MGMT_RING_SIZE - 1;

	/* Allocate RX ring descriptor's memory except Tx ring which allocated eariler */
	pRxDescRing = &pAd->RxDescRing;
	pRxDescRing->AllocSize = RX_RING_SIZE * RXD_SIZE;
	pRxDescRing->AllocVa = (void *) pci_alloc_consistent(pAd->pdev, 
		sizeof(char)*pRxDescRing->AllocSize, &pRxDescRing->AllocPa);
	if (pRxDescRing->AllocVa == NULL)
	{
		DBGPRINT_MAC(DEBUG_ERROR, 
			("RxDescRing Failed to allocate a big buffer\n"));
		return Status = NDIS_STATUS_FAILURE;
	}
	memset(pRxDescRing->AllocVa, 0,pRxDescRing->AllocSize);
	/* Save PA & VA for further operation */
	RingBasePaHigh = 0;
	RingBasePaLow  = pRxDescRing->AllocPa;
	RingBaseVa     = pRxDescRing->AllocVa;
	
	/* Initialize Rx Ring and associated buffer memory */
	pRxRing = &pAd->RxRing;
	for (RxRingIndex = 0; RxRingIndex < RX_RING_SIZE; RxRingIndex++)
	{
		/* Init RX Ring Size, Va, Pa variables */
		pRxRing->Cell[RxRingIndex].AllocSize = RXD_SIZE;
		pRxRing->Cell[RxRingIndex].AllocVa = RingBaseVa;
		pRxRing->Cell[RxRingIndex].AllocPa = RingBasePaLow;

		RingBasePaLow += RXD_SIZE;
		RingBaseVa = (UCHAR *) RingBaseVa + RXD_SIZE;
		
		/* Setup Rx associated Buffer size & allocate share memory */
		pDmaBuf = &pRxRing->Cell[RxRingIndex].DmaBuf;
		pDmaBuf->AllocSize = RX_BUFFER_AGGRESIZE;
		pPacket = dev_alloc_skb(pDmaBuf->AllocSize);
		pDmaBuf->AllocVa = ((struct sk_buff *) pPacket)->data;
		pDmaBuf->AllocPa = pci_map_single(pAd->pdev, 
			pDmaBuf->AllocVa, RX_BUFFER_AGGRESIZE, PCI_DMA_FROMDEVICE);

		pRxRing->Cell[RxRingIndex].pNdisPacket = pPacket;
		if (pDmaBuf->AllocVa == NULL)
		{
			DBGPRINT_MAC(DEBUG_ERROR,
				("Failed to allocate RxRing's 1st buffer\n"));
			break;
		}
		memset(pDmaBuf->AllocVa, 0, pDmaBuf->AllocSize);

		/* Write RxD buffer address & allocated buffer length */
		pRxD = (PRXD_STRUC) pRxRing->Cell[RxRingIndex].AllocVa;
		pRxD->SDP0 = pDmaBuf->AllocPa;
		pRxD->DDONE = 0;
		
	} /* End for loop RX_RING_SIZE */
	pRxRing->RxCpuIdx = RX_RING_SIZE - 1;
	pRxRing->RxSwReadIdx = 0;

	DBGPRINT_MAC(DEBUG_TRACE, ("<---rt2860_init_rings\n"));
	return Status;
} /* End of rt2860_init_rings */


/* 
 *  Free Tx and Rx ring 
 */
static void rt2860_free_rings(struct ieee80211_hw *hw)
{
	RTMP_ADAPTER 	*pAd = hw->priv;
	INT32			TxRingNum, CellNum;
	RTMP_DMACB	   *pCell;
	struct sk_buff *pPacket;
	RTMP_TX_RING   *pTxRing;  
	RTMP_DMABUF    *pTxDescRing;
	PTXD_STRUC	  	pTxD;
	RTMP_MGMT_RING *pMgmtRing;	
	RTMP_DMABUF    *pMgmtDescRing;	
	RTMP_RX_RING   *pRxRing;  
	RTMP_DMABUF    *pRxDescRing;

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_free_rings\n"));

	/* Free Tx Ring Descriptor and associated buffer memory */
	for (TxRingNum=0; TxRingNum < NUM_OF_TX_RING; TxRingNum++)
	{	
		pTxRing = &pAd->TxRing[TxRingNum];	
		for (CellNum=0; CellNum < TX_RING_SIZE; CellNum++)
		{	
			pCell = &pTxRing->Cell[CellNum];
			pTxD = (PTXD_STRUC) (pCell->AllocVa);
			pPacket = pCell->pNdisPacket;
			if (pPacket)
			{
				pci_unmap_single(pAd->pdev,pTxD->SDPtr0, pTxD->SDLen0, 
					PCI_DMA_TODEVICE);
				dev_kfree_skb(pPacket);
			}
			pCell->pNdisPacket = NULL;		
		} /* End for loop TX_RING_SIZE */

		pTxDescRing = &pAd->TxDescRing[TxRingNum];
	   	if (pTxDescRing->AllocVa)
	    	pci_free_consistent(pAd->pdev, pTxDescRing->AllocSize, 
	    		pTxDescRing->AllocVa, pTxDescRing->AllocPa);
	    memset(pTxDescRing, 0, sizeof(RTMP_DMABUF));
	} /* End for loop NUM_OF_TX_RING */
	
	pMgmtRing = &pAd->MgmtRing;
	for (CellNum=0; CellNum < MGMT_RING_SIZE; CellNum++)
	{	
		pCell = &pMgmtRing->Cell[CellNum];
		pTxD = (PTXD_STRUC) (pCell->AllocVa);
		pPacket = pCell->pNdisPacket;
		if (pPacket)
		{
			pci_unmap_single(pAd->pdev,pTxD->SDPtr0, pTxD->SDLen0, 
				PCI_DMA_TODEVICE);
			dev_kfree_skb(pPacket);
		}
		pCell->pNdisPacket = NULL;		
	} /* End for loop MGMT_RING_SIZE */
	pMgmtDescRing = &pAd->MgmtDescRing;
   	if (pMgmtDescRing->AllocVa)
    	pci_free_consistent(pAd->pdev, pMgmtDescRing->AllocSize, 
    		pMgmtDescRing->AllocVa, pMgmtDescRing->AllocPa);
    memset(pMgmtDescRing, 0, sizeof(RTMP_DMABUF));

	/* Free Rx Ring Descriptor and associated buffer memory */
	pRxRing = &pAd->RxRing;
	for (CellNum = 0; CellNum < RX_RING_SIZE; CellNum++)
	{
		pCell = &pRxRing->Cell[CellNum];
		pci_unmap_single(pAd->pdev, pCell->DmaBuf.AllocPa, 
			pCell->DmaBuf.AllocSize, PCI_DMA_FROMDEVICE);
	} /* End for loop RX_RING_SIZE */
	pRxDescRing = &pAd->RxDescRing;
	if (pAd->RxDescRing.AllocVa)
    	pci_free_consistent(pAd->pdev, pAd->RxDescRing.AllocSize, 
    		pAd->RxDescRing.AllocVa, pAd->RxDescRing.AllocPa);
    memset(&pAd->RxDescRing, 0,sizeof(RTMP_DMABUF));

	DBGPRINT_MAC(DEBUG_TRACE, ("<---rt2860_free_rings\n"));
} /* End of rt2860_free_rings */


/* 
 *  Enable Interrupt
 */
static void rt2860_int_enable(PRTMP_ADAPTER pAd, unsigned int mode)
{
	u32 regValue;

	pAd->int_disable_mask &= ~(mode);
	regValue = pAd->int_enable_reg & ~(pAd->int_disable_mask);		
	RTMP_IO_WRITE32(pAd, INT_MASK_CSR, regValue);   // 1:enable
} /* End of rt2860_int_enable */


/* 
 *  Disable Interrupt
 */
static void rt2860_int_disable(PRTMP_ADAPTER pAd, unsigned int mode)
{
	u32 regValue;

	pAd->int_disable_mask |= mode;
	regValue = 	pAd->int_enable_reg & ~(pAd->int_disable_mask);
	RTMP_IO_WRITE32(pAd, INT_MASK_CSR, regValue);   // 0: disable 
} /* End of rt2860_int_disable */


/* 
 *  Handle the Rx. Interrupt
 */
static void rt2860_rxring_handle(struct ieee80211_hw *hw)
{
	RTMP_ADAPTER	*pAd = hw->priv;
	RTMP_RX_RING	*pRxRing;
	unsigned long 	flags;	
	struct sk_buff *skb;
	PRXWI_STRUC		pRxWI;
	UINT32			RxPending = 0;
	RTMP_DMACB 		*pCell;
	PRXD_STRUC		pRxD;
	void			*pRxPacket = NULL;
	void		 	*pNewPacket;
	void		 	*AllocVa;
	dma_addr_t	 	AllocPa;

	pRxRing = &pAd->RxRing;
	RTMP_IO_READ32(pAd, RX_DRX_IDX , &pRxRing->RxDmaIdx);
	if (pRxRing->RxDmaIdx > pRxRing->RxSwReadIdx)
		RxPending = pRxRing->RxDmaIdx - pRxRing->RxSwReadIdx;
	else if (pRxRing->RxDmaIdx < pRxRing->RxSwReadIdx)
		RxPending = pRxRing->RxDmaIdx + RX_RING_SIZE - pRxRing->RxSwReadIdx;

	if (RxPending == 0)
		return;

	DBGPRINT_RX(DEBUG_TRACE, ("RX pending=%d\n",RxPending));
	while (RxPending > 0)
	{	
		struct ieee80211_rx_status rx_status = {0};

		pCell = &pRxRing->Cell[pRxRing->RxSwReadIdx];
		pRxD = (PRXD_STRUC) pCell->AllocVa;	
		if (pRxD->DDONE == 0) {
			DBGPRINT_RX(DEBUG_ERROR, ("Rx handle pRxD->DDONE == 0\n"));
			break;
		}
		pNewPacket = dev_alloc_skb(RX_BUFFER_AGGRESIZE);
		AllocVa = ((struct sk_buff *) pNewPacket)->data;
		AllocPa = pci_map_single(pAd->pdev, AllocVa, RX_BUFFER_AGGRESIZE, 
			PCI_DMA_FROMDEVICE);
		memset(AllocVa, 0, RX_BUFFER_AGGRESIZE);

		if (pNewPacket) {
			spin_lock_irqsave(&pAd->RxRing.Rxlock, flags);
			pci_unmap_single(pAd->pdev, pCell->DmaBuf.AllocPa,
						 pCell->DmaBuf.AllocSize, PCI_DMA_FROMDEVICE);
			pRxPacket = pCell->pNdisPacket;

			pCell->DmaBuf.AllocSize	= RX_BUFFER_AGGRESIZE;
			pCell->pNdisPacket		= (void *) pNewPacket;
			pCell->DmaBuf.AllocVa	= AllocVa;
			pCell->DmaBuf.AllocPa	= AllocPa;
			pRxD->SDP0 = AllocPa;
			pRxD->DDONE = 0;
			spin_unlock_irqrestore(&pAd->RxRing.Rxlock, flags);
		} else {
			DBGPRINT_RX(DEBUG_ERROR, 
				("RX handle cann't allocate the skb space.\n"));
			break;
		}
		
		skb = (struct sk_buff *) pRxPacket;
		pRxWI = (PRXWI_STRUC) skb->data;
		skb_put(skb,pRxWI->MPDUtotalByteCount + RXWI_SIZE);
		skb_pull(skb,RXWI_SIZE);

		rx_status.ssi = max((int) ConvertToRssi(pAd, \
			(CHAR) pRxWI->RSSI0, (UCHAR) 0), (int) ConvertToRssi(pAd, \
			(CHAR) pRxWI->RSSI1, (UCHAR) 1));
		rx_status.ssi = max(rx_status.ssi,(int) ConvertToRssi(pAd, \
			(CHAR) pRxWI->RSSI2, (UCHAR) 2));

		rx_status.signal = ConvertRssiToSignalQuality(rx_status.ssi);
		if(pAd->signalQuality !=0)
			rx_status.signal = 
				(unsigned int)((pAd->signalQuality + rx_status.signal + 1) / 2);

		if (pRxWI->BW == BW_20)
			rx_status.noise = 0x32 + (GET_LNA_GAIN(pAd)*5)/3;
		else if (pRxWI->BW == BW_40)
			rx_status.noise = 0x3A + (GET_LNA_GAIN(pAd)*5)/3;
		rx_status.noise += 256 - 143;

		pAd->signalQuality = rx_status.signal;
		rx_status.freq = channels_2g[pAd->channel - 1].center_freq;
		rx_status.band = IEEE80211_BAND_2GHZ;
		rx_status.rate_idx = pRxWI->MCS + pRxWI->PHYMODE*4;
		if (rx_status.rate_idx >= pAd->band[IEEE80211_BAND_2GHZ].n_bitrates) 
			rx_status.rate_idx = 0;

		ieee80211_rx_irqsafe(hw, skb, &rx_status);

		pRxRing->RxSwReadIdx = (pRxRing->RxSwReadIdx + 1) % RX_RING_SIZE;
		RxPending--;
	}

	pRxRing->RxCpuIdx = (pRxRing->RxSwReadIdx == 0) ? (RX_RING_SIZE-1) : (pRxRing->RxSwReadIdx - 1);
	RTMP_IO_WRITE32(pAd, RX_CRX_IDX, pRxRing->RxCpuIdx);
} /* End of rt2860_rxring_handle */


/* 
 *  Handle the Tx. done Interrupt
 */
static void rt2860_txring_handle(struct ieee80211_hw *hw)
{
	RTMP_ADAPTER 		*pAd = hw->priv;
	TX_STA_FIFO_STRUC	StaFifo;
	UCHAR				pid = 0, wcid = 0, SuccessRate = 0;
	unsigned long 		flags;
	RTMP_DMACB			*pCell;	
	PTXWI_STRUC			pTxWI;	
	PTXD_STRUC			pTxD;	
	struct ieee80211_tx_status tx_status;
	RTMP_TX_RING    	*pTxRing = NULL;
	
	while (1) {
		RTMP_IO_READ32(pAd, TX_STA_FIFO, &StaFifo.word);

		if (StaFifo.field.bValid == 0) {
			break;
		}

		wcid = (UCHAR) StaFifo.field.wcid;
		pid = (UCHAR) StaFifo.field.PidType;
		SuccessRate = (UCHAR) StaFifo.field.SuccessRate;

		pTxRing = &pAd->TxRing[QID_AC_BK];
		spin_lock_irqsave(&pAd->TxRing[QID_AC_BK].Txlock, flags);

		pCell = &pTxRing->Cell[pTxRing->TxDmaIdx];
		pTxWI = (TXWI_STRUC *) pCell->skb->data;

		memset(&tx_status, 0, sizeof(tx_status));			
		while (pTxWI->PacketId != (USHORT) pid) {
			DBGPRINT_TX(DEBUG_TRACE, 
				("Different:TxWI->PacketId=%d,pid= %d\n",pTxWI->PacketId, pid));
			memcpy(&tx_status.control, &pCell->tx_control,sizeof(pCell->tx_control));
			if (!(tx_status.control.flags & IEEE80211_TXCTL_NO_ACK)) {
				tx_status.excessive_retries = 1;
			}
			tx_status.retry_count = pTxWI->MCS;

			pTxD = (PTXD_STRUC) pCell->AllocVa;
			pci_unmap_single(pAd->pdev, pTxD->SDPtr1, pTxD->SDLen1 , PCI_DMA_FROMDEVICE);
			skb_pull(pCell->skb,TXWI_SIZE);
			pTxD->DMADONE = 1;			
			pTxRing->TxDmaIdx = (pTxRing->TxDmaIdx + 1) % TX_RING_SIZE;
			pTxRing->TxSwFreeIdx = (pTxRing->TxSwFreeIdx + 1) % TX_RING_SIZE;
			pAd->lastPacketId = pid;
			ieee80211_tx_status_irqsafe(hw, pCell->skb, &tx_status);

			pCell = &pTxRing->Cell[pTxRing->TxDmaIdx];
			pTxWI = (TXWI_STRUC *) pCell->skb->data;
		}

		if (pTxWI->PacketId == (USHORT) pid) {
			memcpy(&tx_status.control, &pCell->tx_control,sizeof(pCell->tx_control));
			if (!(tx_status.control.flags & IEEE80211_TXCTL_NO_ACK)) {
				if (StaFifo.field.TxSuccess) {
					tx_status.flags = IEEE80211_TX_STATUS_ACK;
				} else {
					tx_status.excessive_retries = 1;
				}
				tx_status.retry_count = pTxWI->MCS - SuccessRate;
				if (tx_status.retry_count > 1) {
					DBGPRINT_TX(DEBUG_TRACE, 
						("PacketId=%d,Request MCS=%d,Send MCS=%d, Rerty=%d\n", \
						pTxWI->PacketId,pTxWI->MCS,SuccessRate,tx_status.retry_count));
				}

			}
			pTxD = (PTXD_STRUC) pCell->AllocVa;
			pci_unmap_single(pAd->pdev, pTxD->SDPtr1, pTxD->SDLen1 , PCI_DMA_FROMDEVICE);
			skb_pull(pCell->skb,TXWI_SIZE);
			pTxD->DMADONE = 1;			
			pTxRing->TxDmaIdx = (pTxRing->TxDmaIdx + 1) % TX_RING_SIZE;
			pTxRing->TxSwFreeIdx = (pTxRing->TxSwFreeIdx + 1) % TX_RING_SIZE;
			pAd->lastPacketId = pid;

			ieee80211_tx_status_irqsafe(hw, pCell->skb, &tx_status);
		}
		
		spin_unlock_irqrestore(&pAd->TxRing[QID_AC_BK].Txlock, flags);

		if (((pAd->lastPacketId % MAX_PacketID) + 1) == pAd->frontPacketId) {
			ieee80211_wake_queue(hw, 0);
			DBGPRINT_TX(DEBUG_TRACE, ("ieee80211_wake_queue\n"));
			}
	}
} /* rt2860_txring_handle */


/* 
 *  Rx done tasklet
 */
void rx_done_task(unsigned long data)
{
	struct ieee80211_hw *hw = (struct ieee80211_hw *) data;
	RTMP_ADAPTER *pAd = hw->priv;

	rt2860_rxring_handle(hw);		
	rt2860_int_enable(pAd, INT_RX_DONE);
} /* rx_done_task */


/* 
 *  Tx done tasklet
 */
void tx_done_task(unsigned long data)
{
	struct ieee80211_hw *hw = (struct ieee80211_hw *) data;
	RTMP_ADAPTER *pAd = hw->priv;

	rt2860_txring_handle(hw);
	rt2860_int_enable(pAd, INT_TX_DONE);
} /* tx_dma_done_task */


/* 
 *  Interrupt handler
 */
static irqreturn_t rt2860_interrupt(int irq, void *dev_id)
{
	struct ieee80211_hw *hw = dev_id;
	RTMP_ADAPTER *pAd = hw->priv;
	INT_SOURCE_CSR_STRUC	IntSource;
	WPDMA_GLO_CFG_STRUC	GloCfg;
	unsigned long flags;
	int				i;
	PTXD_STRUC		pTxD;
	PRXD_STRUC		pRxD;

	/* Inital the Interrupt source. */
	IntSource.word = 0x00000000L;
	RTMP_IO_READ32(pAd, INT_SOURCE_CSR, &IntSource.word);
	RTMP_IO_WRITE32(pAd, INT_SOURCE_CSR, IntSource.word); // write 1 to clear

	if (IntSource.word == 0xffffffff)
        return  IRQ_HANDLED;
	
	if ((IntSource.word & INT_TX_Coherent) || (IntSource.word & INT_RX_Coherent))
	{
		RTMP_IO_READ32(pAd, WPDMA_GLO_CFG , &GloCfg.word);
		GloCfg.field.EnTXWriteBackDDONE = 0;
		GloCfg.field.EnableRxDMA = 0;
		GloCfg.field.EnableTxDMA = 0;
		RTMP_IO_WRITE32(pAd, WPDMA_GLO_CFG, GloCfg.word);

		if (IntSource.word & INT_TX_Coherent) {
			DBGPRINT_MAC(DEBUG_TRACE, ("IntTxCoherent interrupt\n"));
			spin_lock_irqsave(&pAd->TxRing[QID_AC_BK].Txlock, flags);
			for (i=0; i<TX_RING_SIZE; i++)
			{
				pTxD  = (PTXD_STRUC) &pAd->TxRing[QID_AC_BK].Cell[i].AllocVa;
				pTxD->DMADONE = 1 ;
			}
			RTMP_IO_READ32(pAd, TX_DTX_IDX0 + QID_AC_BK*0x10, \
				&pAd->TxRing[QID_AC_BK].TxDmaIdx);
			pAd->TxRing[QID_AC_BK].TxSwFreeIdx = \
				pAd->TxRing[QID_AC_BK].TxDmaIdx - 1;
			pAd->TxRing[QID_AC_BK].TxCpuIdx = pAd->TxRing[QID_AC_BK].TxDmaIdx;
			RTMP_IO_WRITE32(pAd, TX_CTX_IDX0 + QID_AC_BK*0x10, \
				pAd->TxRing[QID_AC_BK].TxCpuIdx);
			spin_unlock_irqrestore(&pAd->TxRing[QID_AC_BK].Txlock, flags);

			spin_lock_irqsave(&pAd->MgmtRing.Mgmtlock, flags);
			for (i=0; i<MGMT_RING_SIZE; i++)
			{
				pTxD  = (PTXD_STRUC) &pAd->MgmtRing.Cell[i].AllocVa;
				pTxD->DMADONE = 1 ;
			}
			RTMP_IO_READ32(pAd, TX_DTX_IDX5, &pAd->MgmtRing.TxDmaIdx);
			pAd->MgmtRing.TxSwFreeIdx = pAd->MgmtRing.TxDmaIdx - 1;
			pAd->MgmtRing.TxCpuIdx = pAd->MgmtRing.TxDmaIdx;
			RTMP_IO_WRITE32(pAd, TX_CTX_IDX5, pAd->MgmtRing.TxCpuIdx);
			spin_unlock_irqrestore(&pAd->MgmtRing.Mgmtlock, flags);
		}

		if (IntSource.word & INT_RX_Coherent) {
			DBGPRINT_MAC(DEBUG_TRACE, ("IntRxCoherent interrupt\n"));
			spin_lock_irqsave(&pAd->RxRing.Rxlock, flags);
			for (i=0; i<RX_RING_SIZE; i++)
			{
				pRxD  = (PRXD_STRUC) &pAd->RxRing.Cell[i].AllocVa;
				pRxD->DDONE = 0 ;
			}
			RTMP_IO_READ32(pAd, RX_DRX_IDX, &pAd->RxRing.RxDmaIdx);
			pAd->RxRing.RxSwReadIdx = pAd->RxRing.RxDmaIdx;
			pAd->RxRing.RxCpuIdx = ((pAd->RxRing.RxDmaIdx == 0) ? \
				(RX_RING_SIZE-1) : (pAd->RxRing.RxDmaIdx-1));
			RTMP_IO_WRITE32(pAd, RX_CRX_IDX, pAd->RxRing.RxCpuIdx);
			spin_unlock_irqrestore(&pAd->RxRing.Rxlock, flags);
		}

		RT28XXDMAEnable(pAd);
		RTMP_IO_WRITE32(pAd, MAC_SYS_CTRL, 0xc);
	} else if (IntSource.word & INT_TX_DONE) {
		DBGPRINT_TX(DEBUG_INFO, ("ISR INT_TX_DONE\n"));
		if ((pAd->int_disable_mask & INT_TX_DONE) == 0) 
		{
			rt2860_int_disable(pAd, INT_TX_DONE);
			tasklet_schedule(&pAd->tx_done_task);
		}	
		pAd->int_pending |= INT_TX_DONE;						
	} else if (IntSource.word & INT_RX_DONE) {
		DBGPRINT_RX(DEBUG_INFO, ("ISR INT_RX_DONE\n"));
		if ((pAd->int_disable_mask & INT_RX_DONE) == 0) 
		{
			rt2860_int_disable(pAd, INT_RX_DONE);
			tasklet_schedule(&pAd->rx_done_task);
		}
		pAd->int_pending |= INT_RX_DONE;
	} else {
	}

	DBGPRINT_MAC(DEBUG_INFO, ("ISR int 0x%08x, mask 0x%08x, pending 0x%08x\n"
		,IntSource.word,pAd->int_disable_mask,pAd->int_pending));
		
	return IRQ_HANDLED;
} /* End of rt2860_interrupt */


static int rt2860_tx(struct ieee80211_hw *hw, struct sk_buff *skb,
		struct ieee80211_tx_control *control)
{
	RTMP_ADAPTER 	*pAd = hw->priv;
	ULONG		 	flags;
	RTMP_TX_RING    *pTxRing = NULL; 	
	RTMP_DMACB 		*pCell;
	struct ieee80211_rate *pRate;
	TXWI_STRUC		*pTxWI;
	PTXD_STRUC		pTxD;
	struct ieee80211_hdr *hdr;
	u16 fc;

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_tx\n"));

	hdr = (struct ieee80211_hdr *)skb->data;
	fc = le16_to_cpu(hdr->frame_control);
	if (DEBUG_TX_Level >= DEBUG_INFO) {
		if ((fc & IEEE80211_FCTL_FTYPE) == IEEE80211_FTYPE_MGMT) {
			if ((fc & IEEE80211_FCTL_STYPE) == IEEE80211_STYPE_ASSOC_REQ) {
				DBGPRINT_TX(DEBUG_INFO, ("Sending AUTH frame\n"));
			} else if ((fc & IEEE80211_FCTL_STYPE) == IEEE80211_STYPE_ASSOC_REQ) {
				DBGPRINT_TX(DEBUG_INFO, ("Sending ASSOC frame\n"));
			} else if ((fc & IEEE80211_FCTL_STYPE) == IEEE80211_STYPE_REASSOC_REQ) {
				DBGPRINT_TX(DEBUG_INFO, ("Sending REASSOC frame\n"));
			}
		}
	}

	while (((pAd->frontPacketId % MAX_PacketID) + 1) == pAd->lastPacketId) /* Id:1 ~ 15 */
	{
		DBGPRINT_TX(DEBUG_TRACE, ("Tx packet wait: frontPacketId=%d, lastPacketId=%d\n",
			pAd->frontPacketId,pAd->lastPacketId));
		RTMPusecDelay(1000);
	}
	spin_lock_irqsave(&pAd->TxRing[QID_AC_BK].Txlock, flags);
	pTxRing = &pAd->TxRing[QID_AC_BK];
	pCell = &pTxRing->Cell[pTxRing->TxCpuIdx];


	pCell->skb = skb;
	memcpy(&pCell->tx_control, control, sizeof(*control));
	skb_push(skb, TXWI_SIZE);

	pTxWI = (TXWI_STRUC *) skb->data;	
	memset(pTxWI,0, TXWI_SIZE);
	pTxWI->MPDUtotalByteCount = skb->len - TXWI_SIZE; 
	pAd->frontPacketId = ((pAd->frontPacketId) % MAX_PacketID) + 1;
	pTxWI->PacketId = pAd->frontPacketId;
	
	pTxWI->ACK = (control->flags & IEEE80211_TXCTL_NO_ACK)? 0:1;
	pTxWI->BW = (control->flags & IEEE80211_TXCTL_40_MHZ_WIDTH)? BW_40:BW_20;
	pTxWI->ShortGI = (control->flags & IEEE80211_TXCTL_SHORT_GI)? 1:0;

	if (control->flags & IEEE80211_TXCTL_FIRST_FRAGMENT)
		pTxWI->FRAG = 1;

	if (control->flags & IEEE80211_TXCTL_AMPDU)
		pTxWI->AMPDU = 1;

	pTxWI->BW = BW_20;
	pRate = control->tx_rate;
	if ((pRate->bitrate == 10) || (pRate->bitrate == 20) || (pRate->bitrate == 55) || (pRate->bitrate == 110)) {
		if (control->tx_rate->flags & IEEE80211_TXCTL_SHORT_PREAMBLE)
			pTxWI->MCS = pRate->hw_value_short;
		else
			pTxWI->MCS = pRate->hw_value;			
		pTxWI->PHYMODE = MODE_CCK;
	} else {
		pTxWI->PHYMODE = MODE_OFDM;
		pTxWI->MCS = pRate->hw_value;
	}

	DBGPRINT_TX(DEBUG_TRACE, ("Dev->xmit[%d bytes] at rate %d\n", skb->len, control->tx_rate->bitrate));
	DBGPRINT_TX(DEBUG_INFO, ("TxWI: FRAG=%d, MMPS=%d, CFACK=%d, TS=%d, AMPDU=%d, MpduDensity=%d, txop=%d\n"
			,pTxWI->FRAG,pTxWI->MIMOps,pTxWI->CFACK,pTxWI->TS,pTxWI->AMPDU,pTxWI->MpduDensity,pTxWI->txop));
	DBGPRINT_TX(DEBUG_INFO, ("TxWI: MCS=%d, BW=%d, ShortGI=%d, STBC=%d, Ifs=%d, TxBF=%d, PHYMODE=%d\n"
			,pTxWI->MCS,pTxWI->BW,pTxWI->ShortGI,pTxWI->STBC,pTxWI->Ifs,pTxWI->TxBF,pTxWI->PHYMODE));
	DBGPRINT_TX(DEBUG_INFO, ("TxWI: ACK=%d, NSEQ=%d, BAWinSize=%d, WCID=%d, MPDUtotalByteCount=%d, PacketId=%d\n"
			,pTxWI->ACK,pTxWI->NSEQ,pTxWI->BAWinSize,pTxWI->WirelessCliID,pTxWI->MPDUtotalByteCount,pTxWI->PacketId));

	pTxD = (PTXD_STRUC) pCell->AllocVa;
	memset(pTxD, 0,TXD_SIZE);	
	pTxD->SDPtr0 = pci_map_single(pAd->pdev, skb->data, skb->len, PCI_DMA_TODEVICE);
	pTxD->SDLen0 = skb->len;
	pTxD->SDLen1 = 0;
	pTxD->LastSec0 = 1;
	pTxD->LastSec1 = 1;
	pTxD->DMADONE = 0;		

	pTxRing->TxCpuIdx = (pTxRing->TxCpuIdx + 1) % TX_RING_SIZE;
	RTMP_IO_WRITE32(pAd, TX_CTX_IDX1, pTxRing->TxCpuIdx)
	spin_unlock_irqrestore(&pAd->TxRing[QID_AC_BK].Txlock, flags);

	if (((pAd->frontPacketId % MAX_PacketID) + 1) == pAd->lastPacketId) { /* Id:1 ~ 15 */
		ieee80211_stop_queue(hw, 0);
		DBGPRINT_TX(DEBUG_TRACE, ("ieee80211_stop_queue\n"));
	}

	DBGPRINT_MAC(DEBUG_TRACE, ("<---rt2860_tx\n"));
	return 0;
} /* End of rt2860_tx */


static int rt2860_start(struct ieee80211_hw *hw)
{
	RTMP_ADAPTER *pAd = hw->priv;
	UINT32 retval;
	UINT32 reg = 0;
	UINT32 MacCsr0 = 0;
	UINT   index;

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_start\n"));

	/* Make sure to get MAC Version */
	index = 0;
	do
	{
		RTMP_IO_READ32(pAd, MAC_CSR0, &MacCsr0);
		pAd->MACVersion = MacCsr0;
		if ((pAd->MACVersion != 0x00) && (pAd->MACVersion != 0xFFFFFFFF))
			break;

		RTMPusecDelay(10);
	} while (index++ < 100);

	/* To fix driver disable/enable hang issue when radio off */
	RTMP_IO_WRITE32(pAd, PWR_PIN_CFG, 0x2);

   	/* If dirver doesn't wake up firmware here,
   	 * NICLoadFirmware will hang forever when interface is up again.
   	 * RT2860 PCI
   	 */
   	RTMP_IO_WRITE32(pAd, AUTO_WAKEUP_CFG, 0);

	/* rx & tx tasklet */
	tasklet_init(&pAd->rx_done_task, rx_done_task, (unsigned long)hw);
	tasklet_init(&pAd->tx_done_task, tx_done_task, (unsigned long)hw);

	/* Setup interrupt handler */
	retval = request_irq(pAd->pdev->irq, &rt2860_interrupt, IRQF_SHARED, "rt2860", hw);
	if (retval) {
		DBGPRINT_MAC(DEBUG_ERROR, 
			("%s: failed to register IRQ handler\n", wiphy_name(hw->wiphy)));
		goto go_exit;
	}

	/* Disable DMA */
	RT28XXDMADisable(pAd);

	/* Load 8051 firmware */
	retval = NICLoadFirmware(pAd);
	if (retval != NDIS_STATUS_SUCCESS)
	{
		DBGPRINT_MAC(DEBUG_ERROR, 
			("NICLoadFirmware failed, Status[=0x%08x]\n", retval));
		goto go_exit;
	}

	/* Disable interrupts */
	RTMP_IO_WRITE32(pAd, INT_MASK_CSR, 0x0);

	/* Initialize tx & rx rings */
	retval = rt2860_init_rings(hw);
	if (retval) {
		DBGPRINT_MAC(DEBUG_ERROR, 
			("%s: failed to initialize rings\n", wiphy_name(hw->wiphy)));
		goto go_exit;
	}

	/* Initialize the hardware */
	RTMP_IO_WRITE32(pAd, WPDMA_RST_IDX, 0x1003f);
	RTMP_IO_WRITE32(pAd, PBF_SYS_CTRL, 0xe1f);
	RTMP_IO_WRITE32(pAd, PBF_SYS_CTRL, 0xe00);

	/* Initialize ASIC */
	retval = NICInitializeAsic(pAd,TRUE);
	if (retval != NDIS_STATUS_SUCCESS)
	{
		DBGPRINT_MAC(DEBUG_ERROR, 
			("NICInitializeAsic failed, Status[=0x%08x]\n", retval));
		goto go_exit;
	}

	// We should read EEPROM for all cases.
	NICInitAsicFromEEPROM(pAd);

	/* 8051 firmware require the signal during booting time. */
	AsicSendCommandToMcu(pAd, 0x72, 0xFF, 0x00, 0x00);

	// Init IRQ parameters
	pAd->int_enable_reg = (0x0|INT_TX_DONE|INT_RX_DONE|INT_RX_Coherent|INT_TX_Coherent);
	pAd->int_disable_mask = 0;
	pAd->int_pending = 0;

	pAd->frontPacketId = MAX_PacketID;
	pAd->lastPacketId = MAX_PacketID;
	pAd->signalQuality = 0;

	// Enable Interrupt
	RTMP_IO_WRITE32(pAd, INT_SOURCE_CSR, 0xffffffff);
	RTMP_IO_WRITE32(pAd, INT_MASK_CSR, pAd->int_enable_reg);

	/* Enable DMA */
	RT28XXDMAEnable(pAd);
	RTMP_IO_WRITE32(pAd, MAC_SYS_CTRL, 0xc);

	/* Clear garbage interrupts */
	RTMP_IO_READ32(pAd, 0x1300, &reg);

go_exit:
	DBGPRINT_MAC(DEBUG_TRACE, ("<---rt2860_start\n"));		
	return retval;
} /* End of rt2860_start */


static void rt2860_stop(struct ieee80211_hw *hw)
{
	RTMP_ADAPTER *pAd = hw->priv;

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_stop\n"));

	/* Disable DMA */
	RT28XXDMADisable(pAd);
	/* Disable interrupts */
	RTMP_IO_WRITE32(pAd, INT_MASK_CSR, 0x0);

	/* Kill all tasklets */
	tasklet_kill(&pAd->tx_done_task);
	tasklet_kill(&pAd->rx_done_task);

	free_irq(pAd->pdev->irq, hw);
	rt2860_free_rings(hw);

	DBGPRINT_MAC(DEBUG_TRACE, ("<---rt2860_stop\n"));	
} /* End of rt2860_stop */


static int rt2860_add_interface(struct ieee80211_hw *hw,
				struct ieee80211_if_init_conf *conf)
{
	RTMP_ADAPTER *pAd = hw->priv;
	DECLARE_MAC_BUF(mac);
	
	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_add_interface\n"));
	
	switch (conf->type) {
		case IEEE80211_IF_TYPE_STA:
			pAd->mode = conf->type;
			break;
		default:
			DBGPRINT_MAC(DEBUG_TRACE, ("<--->rt2860_add_interface\n"));
			return -EOPNOTSUPP;
	}

	RTMP_IO_WRITE32(pAd, MAC_ADDR_DW0,  le32_to_cpu(*(__le32 *)conf->mac_addr));
	RTMP_IO_WRITE32(pAd, MAC_ADDR_DW1,  le32_to_cpu(*(__le32 *)(conf->mac_addr + 4)));
	DBGPRINT_MAC(DEBUG_TRACE, ("Set MAC:%s\n",print_mac(mac, conf->mac_addr)));

	DBGPRINT_MAC(DEBUG_TRACE, ("<--->rt2860_add_interface\n"));
	return 0;
} /* End of rt2860_add_interface */


static void rt2860_remove_interface(struct ieee80211_hw *hw,
				struct ieee80211_if_init_conf *conf)
{
	RTMP_ADAPTER *pAd = hw->priv;

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_remove_interface\n"));

	pAd->mode = IEEE80211_IF_TYPE_MNTR;

	DBGPRINT_MAC(DEBUG_TRACE, ("<---rt2860_remove_interface\n"));
} /* End of rt2860_remove_interface */


static int rt2860_config(struct ieee80211_hw *hw, struct ieee80211_conf *conf)
{
	RTMP_ADAPTER *pAd = hw->priv;
	int channel = ieee80211_frequency_to_channel(conf->channel->center_freq);

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_config\n"));

	if (!conf->radio_enabled) {
		DBGPRINT_MAC(DEBUG_TRACE, ("Radio disabled\n"));
		goto go_exit;
	}

	if (pAd->channel != channel) {
		pAd->channel = channel;
		AsicSwitchChannel(pAd, pAd->channel, FALSE);		

		DBGPRINT_MAC(DEBUG_WARN, ("Channel set to %d[%sGHZ]\n", 
			pAd->channel, conf->channel->band == IEEE80211_BAND_2GHZ ? "2.4" : "5"));
	}

go_exit:
	DBGPRINT_MAC(DEBUG_TRACE, ("<---rt2860_config\n"));
	return 0;
} /* End of rt2860_config */


static int rt2860_config_interface(struct ieee80211_hw *hw,
		struct ieee80211_vif *vif, struct ieee80211_if_conf *conf)
{
	RTMP_ADAPTER *pAd = hw->priv;
	ULONG		  Addr4;

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_config_interface\n"));

	if (conf->bssid) {
		if (memcmp(conf->bssid, pAd->bssid, 6)) {
			memcpy(pAd->bssid, conf->bssid, 6);
			Addr4 = (ULONG)(pAd->bssid[0])		 | 
					(ULONG)(pAd->bssid[1] << 8)  | 
					(ULONG)(pAd->bssid[2] << 16) |
					(ULONG)(pAd->bssid[3] << 24);
			RTMP_IO_WRITE32(pAd, MAC_BSSID_DW0, Addr4);
			Addr4 = 0;
			Addr4 = (ULONG)(pAd->bssid[4]) | (ULONG)(pAd->bssid[5] << 8);
			RTMP_IO_WRITE32(pAd, MAC_BSSID_DW1, Addr4);		
		}
		pAd->ssid_len = conf->ssid_len;
		memcpy(pAd->ssid, conf->ssid, conf->ssid_len);
		DBGPRINT_MAC(DEBUG_WARN, ("Bssid was set to: %s\n", (char *) conf->ssid));	
	}

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_config_interface\n"));
	return 0;
} /* End of rt2860_config_interface */


static void rt2860_configure_filter(struct ieee80211_hw *hw, 
		unsigned int changed_flags,	unsigned int *total_flags, 
		int mc_count, struct dev_mc_list *mclist)
{
	RTMP_ADAPTER *pAd = hw->priv;
	unsigned int new_flags = 0;
	unsigned int filter_flags = 0;
	pAd->rx_filter_flag = 0x00017f9f; /* other:0x00017f97 */

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_configure_filter\n"));

	if (*total_flags & FIF_BCN_PRBRESP_PROMISC) {
			new_flags |= FIF_BCN_PRBRESP_PROMISC;
			filter_flags = 0x00000008;
		}

	pAd->rx_filter_flag &= ~filter_flags;
	*total_flags = new_flags;

	DBGPRINT_MAC(DEBUG_TRACE, ("RX Filter:%x\n", pAd->rx_filter_flag));
	RTMP_IO_WRITE32(pAd, RX_FILTR_CFG, pAd->rx_filter_flag);

	DBGPRINT_MAC(DEBUG_TRACE, ("<---rt2860_configure_filter\n"));
} /* End of rt2860_configure_filter */


static int rt2860_get_stats(struct ieee80211_hw *hw,
			    struct ieee80211_low_level_stats *stats)
{
	RTMP_ADAPTER *pAd = hw->priv;

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_get_stats\n"));
	
	memcpy(stats, &pAd->stats, sizeof(*stats));

	DBGPRINT_MAC(DEBUG_TRACE, ("<---rt2860_get_stats\n"));
	return 0;
} /* End of rt2860_get_stats */


static u64 rt2860_get_tsft(struct ieee80211_hw *hw)
{
	RTMP_ADAPTER *pAd = hw->priv;
	u32 tsftl;
	u64 tsft;

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_get_tsft\n"));

    RTMP_IO_READ32(pAd, TSF_TIMER_DW1, &tsftl);
    RTMP_IO_READ32(pAd, TSF_TIMER_DW0, &tsft);
	tsft <<= 32;
	tsft |= tsftl;

	DBGPRINT_MAC(DEBUG_TRACE, ("<---rt2860_get_tsft\n"));
	return tsft;
} /* End of rt2860_get_tsft */


static int __devinit rt2860_probe(struct pci_dev *pdev, 
		const struct pci_device_id *id)
{
	struct ieee80211_hw *hw;
	RTMP_ADAPTER 	*pAd;
	unsigned int	err, TxRingNum;
    unsigned long	csr_addr;
	DECLARE_MAC_BUF(mac);

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_probe\n"));

	if ((err = pci_enable_device(pdev))) {
		DBGPRINT_MAC(DEBUG_ERROR, ("%s: Cannot enable new PCI device\n", 
			pci_name(pdev)));
		return err;
	}

	if ((err = pci_request_regions(pdev, "rt2860"))) {
		DBGPRINT_MAC(DEBUG_ERROR, ("%s: Cannot obtain PCI resources\n", 
			pci_name(pdev)));
		goto err_disable_pdev;
	}

	/* Enable bus mastering */
	pci_set_master(pdev);	

	/* Allocate hw (mac80211 main struct) and hw->priv (driver private data) */
	hw = ieee80211_alloc_hw(sizeof(*pAd), &rt2860_ops);
	if (!hw) {
		DBGPRINT_MAC(DEBUG_ERROR, ("%s: ieee80211 alloc failed\n", 
			pci_name(pdev)));
		err = -ENOMEM;
		goto err_free_region_pci;
	}
	pAd = hw->priv;
	pAd->pdev = pdev;

    /* Map physical address to virtual address for accessing register */
    csr_addr = (unsigned long) ioremap(pci_resource_start(pdev, 0), 
    pci_resource_len(pdev, 0));
    if (!csr_addr) {
		DBGPRINT_MAC(DEBUG_ERROR, ("%s: ioremap failed, region 0x%lX @ 0x%lX\n", 
			pci_name(pdev), (unsigned long) pci_resource_len(pdev, 0),
			(unsigned long) pci_resource_start(pdev, 0)));
        goto err_free_region_pci;
    }
    pAd->CSRBaseAddress = (unsigned char *) csr_addr;

	/* Set private data */
	SET_IEEE80211_DEV(hw, &pdev->dev);
	pci_set_drvdata(pdev, hw);

	/* Read MAC address from Registry & E2PROM */
	NICReadEEPROMParameters(pAd, (unsigned char *) &pAd->PermanentAddress);
	if (!is_valid_ether_addr(pAd->PermanentAddress)) {
		DBGPRINT_MAC(DEBUG_WARN, ("%s: Invalid hwaddr in EEPROM!\n", 
			pci_name(pdev)));
		random_ether_addr(pAd->PermanentAddress);
	}
	SET_IEEE80211_PERM_ADDR(hw, pAd->PermanentAddress);

	/* Initialize driver private data */
	hw->rate_control_algorithm = NULL;
	hw->flags = 0;
	hw->extra_tx_headroom = TXWI_SIZE;
	hw->channel_change_time = 200;
	hw->queues = 1;
	hw->max_rssi = -29;
	hw->max_noise = -29;
	hw->max_signal = 100; /* link quality (%) */

	/* 2.4GHz ISM band */
	pAd->band[IEEE80211_BAND_2GHZ].channels = 
		(struct ieee80211_channel *) channels_2g;
	pAd->band[IEEE80211_BAND_2GHZ].bitrates = 
		(struct ieee80211_rate *) rates_2g;
	pAd->band[IEEE80211_BAND_2GHZ].n_channels = ARRAY_SIZE(channels_2g);
	pAd->band[IEEE80211_BAND_2GHZ].n_bitrates = ARRAY_SIZE(rates_2g);
	hw->wiphy->bands[IEEE80211_BAND_2GHZ] = &pAd->band[IEEE80211_BAND_2GHZ];
	if (pAd->PhyMode == PHY_11ABG_MIXED) {
		/* TODO 5GHz ISM band */
	}

	if ((err = ieee80211_register_hw(hw))) {
		DBGPRINT_MAC(DEBUG_ERROR, ("%s: Cannot register device\n", 
			pci_name(pdev)));
		goto err_free_desc_hw;
	}
	DBGPRINT_MAC(DEBUG_TRACE,("%s: hwaddr %s, Rev 0x%02x\n", wiphy_name(hw->wiphy), 
		print_mac(mac, hw->wiphy->perm_addr), pdev->revision));

	for (TxRingNum=0; TxRingNum < NUM_OF_TX_RING; TxRingNum++)
		spin_lock_init(&pAd->TxRing[TxRingNum].Txlock);
	
	spin_lock_init(&pAd->MgmtRing.Mgmtlock);
	spin_lock_init(&pAd->RxRing.Rxlock);

	return 0;
	
err_free_desc_hw:
	ieee80211_free_hw(hw);
err_free_region_pci:
	pci_set_drvdata(pdev, NULL);
	pci_release_regions(pdev);
err_disable_pdev:
	pci_disable_device(pdev);

	DBGPRINT_MAC(DEBUG_TRACE, ("<---rt2860_probe\n"));
	return err;
} /* End of rt2860_probe */


static void __devexit rt2860_remove(struct pci_dev *pdev)
{
	struct ieee80211_hw *hw = pci_get_drvdata(pdev);
	RTMP_ADAPTER *pAd;

	DBGPRINT_MAC(DEBUG_TRACE, ("--->rt2860_remove\n"));

	if (!hw)
		goto go_exit;

	ieee80211_unregister_hw(hw);

	pAd = hw->priv;
	pci_release_regions(pdev);
	pci_disable_device(pdev);
	ieee80211_free_hw(hw);

go_exit:	
	DBGPRINT_MAC(DEBUG_TRACE, ("<---rt2860_remove\n"));
} /* End of rt2860_remove */


static int __devinit rt2860_init(void)
{
	return pci_register_driver(&rt2860_pci_driver);
} /* End of rt2860_init */


static void __devexit rt2860_exit(void)
{
	pci_unregister_driver(&rt2860_pci_driver);
} /* End of rt2860_exit */


module_init(rt2860_init);
module_exit(rt2860_exit);

