/*
 *************************************************************************
 * Ralink Tech Inc.
 * 5F., No.36, Taiyuan St., Jhubei City,
 * Hsinchu County 302,
 * Taiwan, R.O.C.
 *
 * (c) Copyright 2002-2007, Ralink Technology, Inc.
 *
 * This program is free software; you can redistribute it and/or modify  * 
 * it under the terms of the GNU General Public License as published by  * 
 * the Free Software Foundation; either version 2 of the License, or     * 
 * (at your option) any later version.                                   * 
 *                                                                       * 
 * This program is distributed in the hope that it will be useful,       * 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         * 
 * GNU General Public License for more details.                          * 
 *                                                                       * 
 * You should have received a copy of the GNU General Public License     * 
 * along with this program; if not, write to the                         * 
 * Free Software Foundation, Inc.,                                       * 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             * 
 *                                                                       * 
 *************************************************************************

    Module Name:
    rt2860pci.h

    Abstract:
	Ralink RT2860 PCI driver based on mac80211 stack

    Revision History:
    Who         When          What
    --------    ----------    ----------------------------------------------
	Eddy Tsai	01-12-2008    Created
*/
#ifndef RT2860PCI_H
#define RT2860PCI_H

#include "rt_config.h"

#define NDIS_STATUS                             INT
#define NDIS_STATUS_SUCCESS                     0x00
#define NDIS_STATUS_FAILURE                     0x01
#define NDIS_STATUS_INVALID_DATA				0x02
#define NDIS_STATUS_RESOURCES                   0x03

/* Device ID & Vendor ID, these values should match EEPROM value */
#define RT2860_PCI_DEVICE_ID   0x0601
#define RT2860_PCIe_DEVICE_ID  0x0681
#define RT_PCI_VENDOR_ID       0x1814


/* EEPROM Related MACRO */
#define RT28xx_EEPROM_READ16(pAd, offset, var)		\
	var = RTMP_EEPROM_READ16(pAd, offset)

#define RT28xx_EEPROM_WRITE16(pAd, offset, var)		\
	RTMP_EEPROM_WRITE16(pAd, offset, var)


/* Hardware beacon definition */
#define HW_BEACON_MAX_COUNT     8 
#define HW_BEACON_OFFSET		0x0200 


/* Interrupt definition */
#define INT_RX_DLY				(1<<0)		// bit 0	
#define INT_TX_DLY				(1<<1)		// bit 1
#define INT_RX_DONE				(1<<2)		// bit 2
#define INT_AC0_DMA_DONE		(1<<3)		// bit 3
#define INT_AC1_DMA_DONE		(1<<4)		// bit 4
#define INT_AC2_DMA_DONE		(1<<5)		// bit 5
#define INT_AC3_DMA_DONE		(1<<6)		// bit 6
#define INT_HCCA_DMA_DONE		(1<<7)		// bit 7
#define INT_MGMT_DMA_DONE		(1<<8)		// bit 8
#define INT_TX_DONE				(1<<13)		// bit 13
#define INT_RX_Coherent			(1<<16)		// bit 16
#define INT_TX_Coherent			(1<<17)		// bit 17


/* Tx definition */
#define MAX_PacketID		15


/* IEEE 80211 rates and channel definition */
static const struct ieee80211_rate rates_2g[] = {
	{ .bitrate =  10, .hw_value = 0, .hw_value_short =  8,
		.flags = IEEE80211_RATE_SHORT_PREAMBLE },
	{ .bitrate =  20, .hw_value = 1, .hw_value_short =  9,
		.flags = IEEE80211_RATE_SHORT_PREAMBLE },
	{ .bitrate =  55, .hw_value = 2, .hw_value_short = 10,
		.flags = IEEE80211_RATE_SHORT_PREAMBLE },
	{ .bitrate = 110, .hw_value = 3, .hw_value_short = 11,
		.flags = IEEE80211_RATE_SHORT_PREAMBLE },
	{ .bitrate =  60, .hw_value = 0, .flags = 0  },
	{ .bitrate =  90, .hw_value = 1, .flags = 0  },
	{ .bitrate = 120, .hw_value = 2, .flags = 0  },
	{ .bitrate = 180, .hw_value = 3, .flags = 0  },
	{ .bitrate = 240, .hw_value = 4, .flags = 0  },
	{ .bitrate = 360, .hw_value = 5, .flags = 0  },
	{ .bitrate = 480, .hw_value = 6, .flags = 0  },
	{ .bitrate = 540, .hw_value = 7, .flags = 0  },
};
static const struct ieee80211_channel channels_2g[] = {
	{ .center_freq = 2412},
	{ .center_freq = 2417},
	{ .center_freq = 2422},
	{ .center_freq = 2427},
	{ .center_freq = 2432},
	{ .center_freq = 2437},
	{ .center_freq = 2442},
	{ .center_freq = 2447},
	{ .center_freq = 2452},
	{ .center_freq = 2457},
	{ .center_freq = 2462},
	{ .center_freq = 2467},
	{ .center_freq = 2472},
	{ .center_freq = 2484},
};


/*
 *  RT2860 register definition
 */
#define MAC_CSR0            0x1000 
#define INT_SOURCE_CSR      0x0200
typedef	union	_INT_SOURCE_CSR_STRUC	{
	struct	{
		UINT32		RxDelayINT:1;
		UINT32		TxDelayINT:1;
		UINT32		RxDone:1;
		UINT32		Ac0DmaDone:1;//4      
		UINT32      Ac1DmaDone:1;
		UINT32      Ac2DmaDone:1;
		UINT32      Ac3DmaDone:1;
		UINT32      HccaDmaDone:1; // bit7
		UINT32      MgmtDmaDone:1;
		UINT32      MCUCommandINT:1;//bit 9
		UINT32      RxTxCoherent:1;
		UINT32      TBTTInt:1;
		UINT32      PreTBTT:1;
		UINT32      TXFifoStatusInt:1;//FIFO Statistics is full, sw should read 0x171c
		UINT32      AutoWakeup:1;//bit14
		UINT32      GPTimer:1;
		UINT32      RxCoherent:1;//bit16
		UINT32      TxCoherent:1;
		UINT32      :14;
	}	field;
	UINT32			word;
} INT_SOURCE_CSR_STRUC, *PINT_SOURCE_CSR_STRUC;

#define INT_MASK_CSR        0x0204
#define WPDMA_GLO_CFG 		0x0208
typedef	union	_WPDMA_GLO_CFG_STRUC	{
	struct	{
		UINT32		EnableTxDMA:1;
		UINT32	  	TxDMABusy:1;
		UINT32		EnableRxDMA:1;
		UINT32		RxDMABusy:1;
		UINT32    	WPDMABurstSIZE:2;
		UINT32    	EnTXWriteBackDDONE:1;
		UINT32    	BigEndian:1;
		UINT32    	RXHdrScater:8;
		UINT32    	HDR_SEG_LEN:16;
	}	field;
	UINT32		word;
} WPDMA_GLO_CFG_STRUC, *PWPDMA_GLO_CFG_STRUC;

#define WPDMA_RST_IDX 		0x020c
#define DELAY_INT_CFG		0x0210
typedef	union	_DELAY_INT_CFG_STRUC	{
	struct	{
		UINT32		RXMAX_PTIME:8;
		UINT32      RXMAX_PINT:7;
		UINT32      RXDLY_INT_EN:1;
		UINT32      TXMAX_PTIME:8;
		UINT32      TXMAX_PINT:7;
		UINT32      TXDLY_INT_EN:1;
	}	field;
	UINT32			word;
} DELAY_INT_CFG_STRUC, *PDELAY_INT_CFG_STRUC;

#define WMM_AIFSN_CFG   	0x0214
#define WMM_CWMIN_CFG   	0x0218
#define WMM_CWMAX_CFG   	0x021c
#define GPIO_CTRL_CFG    	0x0228
#define TX_BASE_PTR0    	0x0230
#define TX_MAX_CNT0      	0x0234
#define TX_CTX_IDX0       	0x0238
#define TX_DTX_IDX0      	0x023c
#define TX_BASE_PTR1     	0x0240
#define TX_MAX_CNT1      	0x0244
#define TX_CTX_IDX1       	0x0248
#define TX_DTX_IDX1      	0x024c
#define TX_BASE_PTR2     	0x0250
#define TX_MAX_CNT2      	0x0254
#define TX_CTX_IDX2       	0x0258
#define TX_DTX_IDX2      	0x025c
#define TX_BASE_PTR3     	0x0260
#define TX_MAX_CNT3      	0x0264
#define TX_CTX_IDX3       	0x0268
#define TX_DTX_IDX3      	0x026c
#define TX_BASE_PTR4     	0x0270
#define TX_MAX_CNT4      	0x0274
#define TX_CTX_IDX4       	0x0278
#define TX_DTX_IDX4      	0x027c
#define TX_BASE_PTR5     	0x0280
#define TX_MAX_CNT5     	0x0284
#define TX_CTX_IDX5       	0x0288
#define TX_DTX_IDX5      	0x028c
#define TX_MGMTMAX_CNT      TX_MAX_CNT5
#define TX_MGMTCTX_IDX      TX_CTX_IDX5
#define TX_MGMTDTX_IDX      TX_DTX_IDX5
#define RX_BASE_PTR     	0x0290
#define RX_MAX_CNT      	0x0294
#define RX_CRX_IDX       	0x0298
#define RX_DRX_IDX      	0x029c
#define PBF_SYS_CTRL 	 	0x0400
#define HOST_CMD_CSR		0x0404
#define PBF_MAX_PCNT	 	0x040C
#define BCN_OFFSET0			0x042C
#define BCN_OFFSET1			0x0430
#define MAC_CSR0            0x1000
#define MAC_SYS_CTRL		0x1004
#define MAC_ADDR_DW0        0x1008
#define MAC_ADDR_DW1    	0x100c
#define MAX_LEN_CFG         0x1018
#define BBP_CSR_CFG			0x101c
typedef	union	_BBP_CSR_CFG_STRUC	{
	struct	{
		UINT32		Value:8;
		UINT32		RegNum:8;
		UINT32		fRead:1;
		UINT32		Busy:1;
		UINT32		BBP_PAR_DUR:1;
		UINT32		BBP_RW_MODE:1;
		UINT32		:12;
	}	field;
	UINT32			word;
} BBP_CSR_CFG_STRUC, *PBBP_CSR_CFG_STRUC;

#define LED_CFG           	0x102c
#define XIFS_TIME_CFG       0x1100
#define BKOFF_SLOT_CFG      0x1104
#define MAC_BSSID_DW0       0x1010
#define MAC_BSSID_DW1       0x1014
#define TSF_TIMER_DW0       0x111C
#define TSF_TIMER_DW1       0x1120
#define MAC_STATUS_CFG		0x1200
#define PWR_PIN_CFG         0x1204
#define AUTO_WAKEUP_CFG     0x1208
#define RF_CSR_CFG0         0x1020	
typedef	union	_PHY_CSR4_STRUC	{
	struct	{
		UINT32		RFRegValue:24;
		UINT32		NumberOfBits:5;
		UINT32		IFSelect:1;		
		UINT32		PLL_LD:1;		
		UINT32		Busy:1;			
	}	field;
	UINT32			word;
}	PHY_CSR4_STRUC, *PPHY_CSR4_STRUC;

#define TX_PIN_CFG			0x1328	
#define TX_BAND_CFG			0x132c
#define TX_SW_CFG0			0x1330
#define TX_SW_CFG1			0x1334
#define TXOP_CTRL_CFG		0x1340
#define TX_RTS_CFG			0x1344
#define TX_RTY_CFG			0x134c
#define TX_TIMEOUT_CFG		0x1348
#define TX_LINK_CFG			0x1350
#define CCK_PROT_CFG		0x1364
#define OFDM_PROT_CFG		0x1368
#define MM20_PROT_CFG		0x136C
#define MM40_PROT_CFG		0x1370
#define GF20_PROT_CFG		0x1374
#define GF40_PROT_CFG		0x1378
#define EXP_CTS_TIME		0x137C
#define EXP_ACK_TIME		0x1380
#define RX_FILTR_CFG		0x1400
#define AUTO_RSP_CFG		0x1404
#define LEGACY_BASIC_RATE	0x1408
#define HT_BASIC_RATE		0x140c
#define TXOP_HLDR_ET		0x1608
#define RX_STA_CNT0			0x1700
#define RX_STA_CNT1			0x1704
#define RX_STA_CNT2			0x1708
#define TX_STA_CNT0			0x170C
#define TX_STA_CNT1			0x1710
#define TX_STA_CNT2			0x1714
#define TX_STA_FIFO			0x1718
typedef	union PACKED _TX_STA_FIFO_STRUC	{
	struct	{
		UINT32     	bValid:1;    // 1:This register contains a valid TX result
		UINT32     	PidType:4;
		UINT32     	TxSuccess:1; // Tx No retry success
		UINT32     	TxAggre:1;   // Tx Retry Success
		UINT32     	TxAckRequired:1;// Tx fail
		UINT32		wcid:8;		//wireless client index
		UINT32		SuccessRate:13;	//include MCS, mode ,shortGI, BW settingSame format as TXWI Word 0 Bit 31-16. 
		UINT32		TxBF:1;
		UINT32		Reserve:2;
	}	field;
	UINT32			word;
}	TX_STA_FIFO_STRUC, *PTX_STA_FIFO_STRUC;

#define H2M_MAILBOX_CSR     0x7010
#define H2M_BBP_AGENT       0x7028

#define MAC_WCID_ATTRIBUTE_BASE	0x6800
#define SHARED_KEY_MODE_BASE    0x7000
#define HW_WCID_ATTRI_SIZE   	4

typedef	union	_HOST_CMD_CSR_STRUC	{
	struct	{
	    UINT32   HostCommand:8;
	    UINT32   Rsv:24;
	}	field;
	UINT32			word;
}	HOST_CMD_CSR_STRUC, *PHOST_CMD_CSR_STRUC;

typedef union  _H2M_MAILBOX_STRUC {
    struct {
        UINT32       LowByte:8;
        UINT32       HighByte:8;
        UINT32       CmdToken:8;
        UINT32       Owner:8;
    }   field;
    UINT32           word;
} H2M_MAILBOX_STRUC, *PH2M_MAILBOX_STRUC;

#define RTMP_IO_READ32(_A, _R, _pV)									\
{																	\
    if ((_A)->bPCIclkOff == FALSE)                                  \
    {                                                               \
		(*_pV = readl((void *)((_A)->CSRBaseAddress + MAC_CSR0)));	\
		(*_pV = readl((void *)((_A)->CSRBaseAddress + (_R))));		\
    }                                                               \
    else															\
		*_pV = 0;													\
}

#define RTMP_IO_WRITE32(_A, _R, _V)								\
{																\
    if ((_A)->bPCIclkOff == FALSE)                              \
    {                                                           \
	UINT32 Val;											        \
	Val = readl((void *)((_A)->CSRBaseAddress + MAC_CSR0));		\
	writel((_V), (void *)((_A)->CSRBaseAddress + (_R)));		\
    }                                                           \
}

#define MAX_BUSY_COUNT  100

#define RTMP_BBP_IO_READ8_BY_REG_ID(_A, _I, _pV)        \
{                                                       \
    BBP_CSR_CFG_STRUC  BbpCsr;                          \
    int             i, k;                               \
    if ((_A)->bPCIclkOff == FALSE)                      \
    {                                                   \
    for (i=0; i<MAX_BUSY_COUNT; i++)                    \
    {                                                   \
		RTMP_IO_READ32(_A, H2M_BBP_AGENT, &BbpCsr.word);\
        if (BbpCsr.field.Busy == BUSY)                  \
        {                                               \
            continue;                                   \
        }                                               \
        BbpCsr.word = 0;                                \
        BbpCsr.field.fRead = 1;                         \
        BbpCsr.field.BBP_RW_MODE = 1;                   \
        BbpCsr.field.Busy = 1;                          \
        BbpCsr.field.RegNum = _I;                       \
		RTMP_IO_WRITE32(_A, H2M_BBP_AGENT, BbpCsr.word);\
		AsicSendCommandToMcu(_A, 0x80, 0xff, 0x0, 0x0);	\
		RTMPusecDelay(1000);							\
        for (k=0; k<MAX_BUSY_COUNT; k++)                \
        {                                               \
			RTMP_IO_READ32(_A, H2M_BBP_AGENT, &BbpCsr.word);\
            if (BbpCsr.field.Busy == IDLE)              \
                break;                                  \
        }                                               \
        if ((BbpCsr.field.Busy == IDLE) &&              \
            (BbpCsr.field.RegNum == _I))                \
        {                                               \
            *(_pV) = (UCHAR)BbpCsr.field.Value;         \
            break;                                      \
        }                                               \
    }                                                   \
    if (BbpCsr.field.Busy == BUSY)                      \
    {                                                   \
		printk(KERN_INFO "BBP read R%d=0x%x fail\n", _I, BbpCsr.word);\
        *(_pV) = (_A)->BbpWriteLatch[_I];               \
		RTMP_IO_READ32(_A, H2M_BBP_AGENT, &BbpCsr.word);\
		BbpCsr.field.Busy = 0;                          \
		RTMP_IO_WRITE32(_A, H2M_BBP_AGENT, BbpCsr.word);\
    }                                                   \
    }                   								\
}

#define RTMP_BBP_IO_WRITE8_BY_REG_ID(_A, _I, _V)        \
{                                                       \
    BBP_CSR_CFG_STRUC  BbpCsr;                          \
    int             BusyCnt;                            \
    if ((_A)->bPCIclkOff == FALSE)                      \
    {                                                   \
    for (BusyCnt=0; BusyCnt<MAX_BUSY_COUNT; BusyCnt++)  \
    {                                                   \
		RTMP_IO_READ32(_A, H2M_BBP_AGENT, &BbpCsr.word);\
        if (BbpCsr.field.Busy == BUSY)                  \
            continue;                                   \
        BbpCsr.word = 0;                                \
        BbpCsr.field.fRead = 0;                         \
        BbpCsr.field.BBP_RW_MODE = 1;                   \
        BbpCsr.field.Busy = 1;                          \
        BbpCsr.field.Value = _V;                        \
        BbpCsr.field.RegNum = _I;                       \
		RTMP_IO_WRITE32(_A, H2M_BBP_AGENT, BbpCsr.word);\
		AsicSendCommandToMcu(_A, 0x80, 0xff, 0x0, 0x0);	\
        (_A)->BbpWriteLatch[_I] = _V;                   \
        break;                                          \
    }                                                   \
    if (BusyCnt == MAX_BUSY_COUNT)                      \
    {                                                   \
		printk(KERN_ERR "BBP read R%d=0x%x fail\n", _I, BbpCsr.word);	\
		RTMP_IO_READ32(_A, H2M_BBP_AGENT, &BbpCsr.word);\
		BbpCsr.field.Busy = 0;                          \
		RTMP_IO_WRITE32(_A, H2M_BBP_AGENT, BbpCsr.word);\
    }                                                   \
    }                                                   \
}

#define RTMP_RF_IO_WRITE32(_A, _V)                  \
{                                                   \
    PHY_CSR4_STRUC  Value;                          \
    ULONG           BusyCnt = 0;                    \
    if ((_A)->bPCIclkOff) 	                		\
    {												\
        return;										\
    }                                               \
    do {                                            \
        RTMP_IO_READ32(_A, RF_CSR_CFG0, &Value.word);  \
        if (Value.field.Busy == IDLE)               \
            break;                                  \
        BusyCnt++;                                  \
    }   while (BusyCnt < MAX_BUSY_COUNT);           \
    if (BusyCnt < MAX_BUSY_COUNT)                   \
    {                                               \
        RTMP_IO_WRITE32(_A, RF_CSR_CFG0, _V);       \
    }                                               \
}
	
#endif /* RT2860PCI_H */

