/*
 *************************************************************************
 * Ralink Tech Inc.
 * 5F., No.36, Taiyuan St., Jhubei City,
 * Hsinchu County 302,
 * Taiwan, R.O.C.
 *
 * (c) Copyright 2002-2007, Ralink Technology, Inc.
 *
 * This program is free software; you can redistribute it and/or modify  * 
 * it under the terms of the GNU General Public License as published by  * 
 * the Free Software Foundation; either version 2 of the License, or     * 
 * (at your option) any later version.                                   * 
 *                                                                       * 
 * This program is distributed in the hope that it will be useful,       * 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        * 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         * 
 * GNU General Public License for more details.                          * 
 *                                                                       * 
 * You should have received a copy of the GNU General Public License     * 
 * along with this program; if not, write to the                         * 
 * Free Software Foundation, Inc.,                                       * 
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             * 
 *                                                                       * 
 *************************************************************************

    Module Name:
    rtmp.h

    Abstract:
    Generic type and structure definition


    Revision History:
    Who         When          What
    --------    ----------    ----------------------------------------------
	Eddy Tsai	01-12-2008    Created
*/
#ifndef RTMP_H
#define RTMP_H

#include	"rt_config.h"

/* Generic definition */
#define PACKED  __attribute__ ((packed))
#define TRUE				1
#define FALSE				0
#define	BUSY		        1
#define	IDLE		        0


/* PHY mode definition */
#define MODE_CCK	0
#define MODE_OFDM   1
typedef enum _RT_802_11_PHY_MODE {
	PHY_11BG_MIXED = 0,
	PHY_11B,
	PHY_11A,
	PHY_11ABG_MIXED,
	PHY_11G,
} RT_802_11_PHY_MODE;


/* Bandwidth definition */
#define BAND_WIDTH_20		0
#define BAND_WIDTH_40		1
#define BAND_WIDTH_BOTH		2
#define BAND_WIDTH_10		3
#define BW_20		BAND_WIDTH_20
#define BW_40		BAND_WIDTH_40
#define BW_BOTH		BAND_WIDTH_BOTH
#define BW_10		BAND_WIDTH_10


/* Ralink chipsets definition */
#define RALINK_2883_VERSION		((UINT32)0x28830300)
#define RALINK_2880E_VERSION	((UINT32)0x28720200)
#define RALINK_3070_VERSION		((UINT32)0x30700200)


/* RFIC type definition */
#define RFIC_2820			1       // 2.4G 2T3R
#define RFIC_2850			2       // 2.4G/5G 2T3R
#define RFIC_2720			3       // 2.4G 1T2R
#define RFIC_2750			4       // 2.4G/5G 1T2R
#define RFIC_3020			5       // 2.4G 1T1R
#define RFIC_2020			6       // 2.4G B/G
typedef struct  _RTMP_RF_REGS
{
	UCHAR   Channel;
	ULONG   R1;
	ULONG   R2;
	ULONG   R3;
	ULONG   R4;
} RTMP_RF_REGS, *PRTMP_RF_REGS;


/* FIRMWARE definition */
#define FIRMWARE_IMAGE_BASE     0x2000


/* EEPROM definition */
#define E2PROM_CSR          	0x0004
#define EEDO                	0x08
#define EEDI                	0x04
#define EECS                	0x02
#define EESK                	0x01
#define EERL                	0x80
#define EEPROM_WRITE_OPCODE 	0x05
#define EEPROM_READ_OPCODE  	0x06
#define	NUM_EEPROM_BBP_PARMS	19
#define	EEPROM_NIC1_OFFSET      0x34
#define	EEPROM_NIC2_OFFSET      0x36
#define EEPROM_LNA_OFFSET		0x44
#define EEPROM_RSSI_BG_OFFSET	0x46
#define EEPROM_RSSI_A_OFFSET	0x4a
#define	EEPROM_G_TX_PWR_OFFSET	0x52
#define	EEPROM_G_TX2_PWR_OFFSET	0x60
#define EEPROM_A_TX_PWR_OFFSET  0x78
#define EEPROM_A_TX2_PWR_OFFSET 0xa6

typedef	union _EEPROM_TX_PWR_STRUC {
	struct	{
		CHAR	Byte0;		// Low Byte
		CHAR	Byte1;		// High Byte
	}	field;
	USHORT	word;
} EEPROM_TX_PWR_STRUC, *PEEPROM_TX_PWR_STRUC;

typedef	union _EEPROM_ANTENNA_STRUC	{
	struct	{
		USHORT	RxPath:4;	// 1: 1R, 2: 2R, 3: 3R
		USHORT	TxPath:4;	// 1: 1T, 2: 2T
		USHORT  RfIcType:4;
		USHORT  Rsv:4;              
	}	field;
	USHORT			word;
} EEPROM_ANTENNA_STRUC, *PEEPROM_ANTENNA_STRUC;

typedef	union _EEPROM_NIC_CINFIG2_STRUC	{
	struct {
		USHORT		HardwareRadioControl:1;	// 1:enable, 0:disable
		USHORT		DynamicTxAgcControl:1;	
		USHORT		ExternalLNAForG:1;		
		USHORT		ExternalLNAForA:1;		// external LNA enable for 2.4G
		USHORT		CardbusAcceleration:1;	// !!! NOTE: 0 - enable, 1 - disable		
		USHORT		BW40MSidebandForG:1;
		USHORT		BW40MSidebandForA:1;
		USHORT		EnableWPSPBC:1;         // WPS PBC Control bit
		USHORT		BW40MAvailForG:1;		// 0:enable, 1:disable
		USHORT		BW40MAvailForA:1;		// 0:enable, 1:disable
		USHORT		Rsv2:6;                 // must be 0
	}	field;
	USHORT			word;
} EEPROM_NIC_CONFIG2_STRUC, *PEEPROM_NIC_CONFIG2_STRUC;


/* Channel and Tx power definition */
#define DEFAULT_RF_TX_POWER     5
#define MAX_NUM_OF_CHANNELS		49 //14 channels @2.4G +  12@UNII + 4 @MMAC + 11 @HiperLAN2 + 7 @Japan + 1 as NULL termination
typedef struct _CHANNEL_TX_POWER {
	USHORT	RemainingTimeForUse; //unit: sec
	UCHAR	Channel;
	UCHAR   bEffectedChannel;	 //For BW 40 operating in 2.4GHz , the "effected channel" is the channel that is covered in 40Mhz.
	CHAR    Power;
	CHAR    Power2;
	UCHAR   MaxTxPwr;
	UCHAR   DfsReq;
} CHANNEL_TX_POWER, *PCHANNEL_TX_POWER;


/* Tx and Rx buffer definition */
#define TXD_SIZE                16
#define TXWI_SIZE               16
#define RXD_SIZE               	16
#define RXWI_SIZE             	16
#define TX_RING_SIZE            64 
#define MGMT_RING_SIZE          128
#define RX_RING_SIZE            128
#define RX_BUFFER_AGGRESIZE     3840
#define MAX_AGGREGATION_SIZE    3840 //3904 //3968 //4096

#define QID_AC_BE               0   // meet ACI definition in 802.11e
#define QID_AC_BK               1   // meet ACI definition in 802.11e
#define QID_AC_VI               2
#define QID_AC_VO               3
#define QID_HCCA                4
#define NUM_OF_TX_RING          5
#define QID_MGMT                6
#define QID_RX                  7

typedef	struct	PACKED _TXWI_STRUC {
	// Word	0
	UINT32		FRAG:1;		// 1 to inform TKIP engine this is a fragment.
	UINT32		MIMOps:1;	// the remote peer is in dynamic MIMO-PS mode
	UINT32		CFACK:1;
	UINT32		TS:1;		
	UINT32		AMPDU:1;
	UINT32		MpduDensity:3;
	UINT32		txop:2;	//FOR "THIS" frame. 0:HT TXOP rule , 1:PIFS TX ,2:Backoff, 3:sifs only when previous frame exchange is successful.
	UINT32		rsv:6;	
	UINT32		MCS:7;
	UINT32		BW:1;	//channel bandwidth 20MHz or 40 MHz
	UINT32		ShortGI:1;
	UINT32		STBC:2;	// 1: STBC support MCS =0-7,   2,3 : RESERVE
	UINT32		Ifs:1;	// 
	UINT32		rsv2:1;
	UINT32		TxBF:1;	// 3*3
	UINT32		PHYMODE:2;  
	// Word	1
	UINT32		ACK:1;
	UINT32		NSEQ:1;
	UINT32		BAWinSize:6;
	UINT32		WirelessCliID:8;
	UINT32		MPDUtotalByteCount:12;
	UINT32		PacketId:4;
	//Word 2
	UINT32		IV;
	//Word 3
	UINT32		EIV;
}	TXWI_STRUC, *PTXWI_STRUC;

typedef	struct	PACKED _RXWI_STRUC {
	// Word	0
	UINT32		WirelessCliID:8;
	UINT32		KeyIndex:2;
	UINT32		BSSID:3;
	UINT32		UDF:3;
	UINT32		MPDUtotalByteCount:12;
	UINT32		TID:4;
	// Word	1
	UINT32		FRAG:4;
	UINT32		SEQUENCE:12;
	UINT32		MCS:7;
	UINT32		BW:1;
	UINT32		ShortGI:1;
	UINT32		STBC:2;
	UINT32		rsv:3;
	UINT32		PHYMODE:2;              // 1: this RX frame is unicast to me
	//Word 2
	UINT32		RSSI0:8;
	UINT32		RSSI1:8;
	UINT32		RSSI2:8;
	UINT32		rsv1:8;
	//Word 3
	UINT32		SNR0:8;
	UINT32		SNR1:8;
	UINT32		rsv2:16;
}	RXWI_STRUC, *PRXWI_STRUC;

typedef	struct	PACKED _TXD_STRUC {
	// Word	0
	UINT32		SDPtr0;
	// Word	1
	UINT32		SDLen1:14;
	UINT32		LastSec1:1;
	UINT32		Burst:1;
	UINT32		SDLen0:14;
	UINT32		LastSec0:1;
	UINT32		DMADONE:1;
	//Word 2
	UINT32		SDPtr1;
	//Word 3
	UINT32		rsv2:24;
	UINT32		WIV:1;	// Wireless Info Valid. 1 if Driver already fill WI,  o if DMA needs to copy WI to correctposition
	UINT32		QSEL:2;	// select on-chip FIFO ID for 2nd-stage output scheduler.0:MGMT, 1:HCCA 2:EDCA
	UINT32		rsv:2;
	UINT32		TCO:1;
	UINT32		UCO:1;
	UINT32		ICO:1;
}	TXD_STRUC, *PTXD_STRUC;

typedef	struct	PACKED _RXD_STRUC	{
	// Word	0
	UINT32		SDP0;
	// Word	1
	UINT32		SDL1:14;
	UINT32		Rsv:2;
	UINT32		SDL0:14;
	UINT32		LS0:1;
	UINT32		DDONE:1;
	// Word	2
	UINT32		SDP1;
	// Word	3
	UINT32		BA:1;
	UINT32		DATA:1;
	UINT32		NULLDATA:1;
	UINT32		FRAG:1;
	UINT32		U2M:1;       // 1: this RX frame is unicast to me
	UINT32		Mcast:1;     // 1: this is a multicast frame
	UINT32		Bcast:1;     // 1: this is a broadcast frame	
	UINT32		MyBss:1;  	 // 1: this frame belongs to the same BSSID	
	UINT32		Crc:1;       // 1: CRC error
	UINT32		CipherErr:2; // 0: decryption okay, 1:ICV error, 2:MIC error, 3:KEY not valid
	UINT32		AMSDU:1;	 // rx with 802.3 header, not 802.11 header.
	UINT32		HTC:1;
	UINT32		RSSI:1;
	UINT32		L2PAD:1;
	UINT32		AMPDU:1;	
	UINT32		Decrypted:1; // this frame is being decrypted.
	UINT32		PlcpSignal:1;// To be moved
	UINT32		PlcpRssil:1; // To be moved
	UINT32		Rsv1:13;
}	RXD_STRUC, *PRXD_STRUC, RT28XX_RXD_STRUC, *PRT28XX_RXD_STRUC;

typedef struct  _RTMP_DMABUF
{
	ULONG		AllocSize;
	void        *AllocVa;    // TxBuf virtual address
	dma_addr_t	AllocPa;     // TxBuf physical address
} RTMP_DMABUF, *PRTMP_DMABUF;

typedef struct _RTMP_DMACB
{
	ULONG           AllocSize;   // Control block size
	void            *AllocVa;    // Control block virtual address
	dma_addr_t		AllocPa;     // Control block physical address
	void			*pNdisPacket;
	void 			*pNextNdisPacket;
	RTMP_DMABUF		DmaBuf;

	TXWI_STRUC		*pTxWI;
	struct sk_buff	 *skb;
	struct ieee80211_tx_control tx_control;	
} RTMP_DMACB, *PRTMP_DMACB;

typedef struct _RTMP_TX_RING
{
	RTMP_DMACB  Cell[TX_RING_SIZE];
	UINT32		TxCpuIdx;	
	UINT32		TxDmaIdx;   
	UINT32		TxSwFreeIdx; 	// software next free tx index	
	spinlock_t 	Txlock;	
} RTMP_TX_RING, *PRTMP_TX_RING;

typedef struct _RTMP_RX_RING
{
	RTMP_DMACB  Cell[RX_RING_SIZE];
	UINT32		RxCpuIdx;
	UINT32		RxDmaIdx;
	INT32		RxSwReadIdx; 	// software next read index
	spinlock_t 	Rxlock;	
} RTMP_RX_RING, *PRTMP_RX_RING;

typedef struct _RTMP_MGMT_RING
{
	RTMP_DMACB  Cell[MGMT_RING_SIZE];
	UINT32		TxCpuIdx;
	UINT32		TxDmaIdx;
	UINT32		TxSwFreeIdx; // software next free tx index
	spinlock_t 	Mgmtlock;	
} RTMP_MGMT_RING, *PRTMP_MGMT_RING;


/*
 *  The Ralink adapter structure
 */
typedef struct _RTMP_ADAPTER
{
	UINT32          MACVersion;
	
	/* PCI related parameters */
	struct pci_dev 	*pdev;
   	UCHAR			bPCIclkOff;			//flag that indicate if the PICE power status in Configuration SPace..
	UCHAR			*CSRBaseAddress;	//PCI MMIO Base Address
	UINT			int_enable_reg;
	UINT			int_disable_mask;
	UINT			int_pending;

	/* RFIC control */
	UCHAR                   	RfIcType;       // RFIC_xxx
	ULONG                   	RfFreqOffset;   // Frequency offset for channel switching
	RTMP_RF_REGS            	LatchRfRegs;    // latch th latest RF programming value since RF IC doesn't support READ
	EEPROM_ANTENNA_STRUC    	Antenna;        // Since ANtenna definition is different for a & g. We need to save it for future reference.
	EEPROM_NIC_CONFIG2_STRUC    NicConfig2;
	CHANNEL_TX_POWER TxPower[MAX_NUM_OF_CHANNELS]; //Store Tx power value for all channels.

	/* E2PROM */
	UCHAR    EEPROMAddressNum;       	// 93c46=6  93c66=8
	USHORT   EEPROMDefaultValue[NUM_EEPROM_BBP_PARMS];
	UCHAR    EepromAccess;

	/* BBP Control */
	UCHAR           BbpWriteLatch[140];     // record last BBP register value written via BBP_IO_WRITE/BBP_IO_WRITE_VY_REG_ID

	/* IEEE80211 */
	struct ieee80211_low_level_stats stats;	
	struct ieee80211_supported_band band[IEEE80211_NUM_BANDS];
	UCHAR   PhyMode;            		// PHY_11A, PHY_11B, PHY_11BG_MIXED, PHY_ABG_MIXED
	UCHAR	PermanentAddress[6];		//Factory default MAC address
	UCHAR   bssid[6];
	u8 		ssid[32];
	size_t 	ssid_len;
	int 	mode;
	int 	channel;
	UCHAR	BBPCurrentBW;

	/* TX.RX Ring */
	RTMP_DMABUF             TxDescRing[NUM_OF_TX_RING];	// Shared memory for Tx descriptors
	RTMP_TX_RING            TxRing[NUM_OF_TX_RING];		// AC0~4 + HCCA
	RTMP_DMABUF             RxDescRing;                 // Shared memory for RX descriptors
	RTMP_RX_RING            RxRing;
	RTMP_DMABUF             MgmtDescRing;              	// Shared memory for MGMT descriptors
	RTMP_MGMT_RING          MgmtRing;
	UINT32 					rx_filter_flag;
	USHORT 					frontPacketId;
	USHORT 					lastPacketId;


	struct tasklet_struct 	rx_done_task;
	struct tasklet_struct 	tx_done_task;

	/* This soft Rx Antenna Diversity mechanism is used only when user set */
	UINT32 		signalQuality;
	BOOLEAN     bAutoTxAgcA;                // Enable driver auto Tx Agc control
	UCHAR	    TssiRefA;					// Store Tssi reference value as 25 temperature.	
	UCHAR	    TssiPlusBoundaryA[5];		// Tssi boundary for increase Tx power to compensate.
	UCHAR	    TssiMinusBoundaryA[5];		// Tssi boundary for decrease Tx power to compensate.
	UCHAR	    TxAgcStepA;					// Store Tx TSSI delta increment / decrement value
	CHAR		TxAgcCompensateA;			// Store the compensation (TxAgcStep * (idx-1))
	
	BOOLEAN     bAutoTxAgcG;                // Enable driver auto Tx Agc control
	UCHAR	    TssiRefG;					// Store Tssi reference value as 25 temperature.	
	UCHAR	    TssiPlusBoundaryG[5];		// Tssi boundary for increase Tx power to compensate.
	UCHAR	    TssiMinusBoundaryG[5];		// Tssi boundary for decrease Tx power to compensate.
	UCHAR	    TxAgcStepG;					// Store Tx TSSI delta increment / decrement value
	CHAR		TxAgcCompensateG;			// Store the compensation (TxAgcStep * (idx-1))

	CHAR		BGRssiOffset0;				// Store B/G RSSI#0 Offset value on EEPROM 0x46h
	CHAR		BGRssiOffset1;				// Store B/G RSSI#1 Offset value 
	CHAR		BGRssiOffset2;				// Store B/G RSSI#2 Offset value 

	CHAR		ARssiOffset0;				// Store A RSSI#0 Offset value on EEPROM 0x4Ah
	CHAR		ARssiOffset1;				// Store A RSSI#1 Offset value 
	CHAR		ARssiOffset2;				// Store A RSSI#2 Offset value 

	CHAR		BLNAGain;					// Store B/G external LNA#0 value on EEPROM 0x44h
	CHAR		ALNAGain0;					// Store A external LNA#0 value for ch36~64
	CHAR		ALNAGain1;					// Store A external LNA#1 value for ch100~128
	CHAR		ALNAGain2;					// Store A external LNA#2 value for ch132~165		
} RTMP_ADAPTER, *PRTMP_ADAPTER;


/* MACRO definition */
#define GET_LNA_GAIN(_pAd)	((_pAd->LatchRfRegs.Channel <= 14) ? (_pAd->BLNAGain) : ((_pAd->LatchRfRegs.Channel <= 64) ? (_pAd->ALNAGain0) : ((_pAd->LatchRfRegs.Channel <= 128) ? (_pAd->ALNAGain1) : (_pAd->ALNAGain2))))

/* EEPROM function */
void RTMPusecDelay(ULONG usec);
void NICReadEEPROMParameters(PRTMP_ADAPTER pAd, UCHAR *mac_addr);

/* RTMP function */
void RT28XXDMADisable(RTMP_ADAPTER *pAd);
void RT28XXDMAEnable(RTMP_ADAPTER *pAd);
UINT NICLoadFirmware(PRTMP_ADAPTER pAd);
void NICInitAsicFromEEPROM(PRTMP_ADAPTER pAd);
void AsicSwitchChannel(PRTMP_ADAPTER pAd, int Channel,BOOLEAN bScan);
BOOLEAN AsicSendCommandToMcu(PRTMP_ADAPTER pAd,UCHAR Command,UCHAR Token,UCHAR Arg0,UCHAR Arg1);
int	NICInitializeAsic(PRTMP_ADAPTER pAd, BOOLEAN bHardReset);
CHAR ConvertToRssi(PRTMP_ADAPTER pAd, CHAR Rssi, UCHAR RssiNumber);
UINT ConvertRssiToSignalQuality(UINT rssi);
#endif /* RTMP_H */

